package dev.pfaff.altargui;

import dev.pfaff.altargui.util.OpenArrayList;

import java.util.Objects;

public class GuiException extends RuntimeException {
	private static final String[] EMPTY_STRING_ARRAY = new String[0];

	private final OpenArrayList<String> messages = OpenArrayList.wrap(EMPTY_STRING_ARRAY);

	public GuiException(String message) {
		super(Objects.requireNonNull(message));
	}

	private GuiException(String message, Throwable cause) {
		super(Objects.requireNonNull(message), unwrapCause(cause));
	}

	private static Throwable unwrapCause(Throwable cause) {
		while (cause.getClass() == RuntimeException.class && cause.getMessage() == null && cause.getCause() != null) {
			cause = cause.getCause();
		}
		return cause;
	}

	public static GuiException wrap(String message, Throwable cause) {
		if (!(cause instanceof GuiException)) {
			cause = new GuiException("Caught exception thrown", cause);
		}
		var e = (GuiException) cause;
		e.messages.add(Objects.requireNonNull(message));
		return e;
	}

	@Override
	public final String getMessage() {
		var initialMessage = super.getMessage();
		var stackHeader = "\n\tStack (root down):";
		var stackEntryPrefix = "\n\t\t";
		var sb = new StringBuilder(initialMessage.length() +
								   stackHeader.length() +
								   this.messages.size() * stackEntryPrefix.length())
			.append(initialMessage)
			.append(stackHeader);
		this.messages.forEachReversed(message -> {
			message.lines().forEach(line -> {
				sb.ensureCapacity(stackEntryPrefix.length() + line.length());
				sb.append(stackEntryPrefix).append(line);
			});
		});
		return sb.toString();
	}
}
