package dev.pfaff.altargui;

import dev.pfaff.altargui.texture.LoadableTexture;
import dev.pfaff.altargui.types.Tex2d;

import java.util.Objects;

public record LoadableTex2d(LoadableTexture tex, Tex2d coords) {
	public LoadableTex2d {
		Objects.requireNonNull(tex);
		Objects.requireNonNull(coords);
	}

	/**
	 * Returns a new instance specifying the entire region of the provided texture.
	 */
	public LoadableTex2d(LoadableTexture tex) {
		this(tex, Tex2d.WHOLE);
	}

	public static LoadableTex2d fromPixelCoords(LoadableTexture tex,
												int u1_i, int v1_i,
												int u2_i, int v2_i,
												int textureWidth, int textureHeight) {
		return new LoadableTex2d(tex, Tex2d.fromPixelCoords(u1_i, v1_i, u2_i, v2_i, textureWidth, textureHeight));
	}

	public static LoadableTex2d fromPixelExtent(LoadableTexture tex,
												int u, int v,
												int uExtent, int vExtent,
												int textureWidth, int textureHeight) {
		return new LoadableTex2d(tex, Tex2d.fromPixelExtent(u, v, uExtent, vExtent, textureWidth, textureHeight));
	}
}
