package dev.pfaff.altargui;

import dev.pfaff.altargui.resource.ClassResourceLoadableResource;
import dev.pfaff.altargui.shader.ProgramLoader;
import dev.pfaff.altargui.shader.ShaderLoader;
import dev.pfaff.altargui.shader.ShaderProg;
import dev.pfaff.altargui.shader.ShaderProg2D;
import dev.pfaff.altargui.util.FakeConstantBoolean;
import dev.pfaff.altargui.util.OpenArrayList;

import java.util.List;
import java.util.function.Supplier;

import static dev.pfaff.altargui.GuiGlobals.LOGGER;
import static dev.pfaff.altargui.internal.InternalGlobals.GL_FEATURES;
import static dev.pfaff.altargui.util.ThrowUtil.throwUnchecked;
import static dev.pfaff.log4truth.StandardTags.ERROR;

public final class Shaders {
	private static final FakeConstantBoolean initialized = new FakeConstantBoolean();
	public static boolean isInitialized() {
		return initialized.get();
	}

	private static final ShaderLoader loader = new ShaderLoader(name -> {
		return new ClassResourceLoadableResource(Shaders.class, "/assets/altar-gui/" + name);
	});

	private static ShaderProg2D rectSolidColorInstancedShader;
	private static ShaderProg2D rectSolidTexColorInstancedShader;
	private static ShaderProg2D textInstancedShader;
	private static ShaderProg2D genericInstancedShader;

	public static final Supplier<ShaderProg2D> RECT_SOLID_COLOR_INSTANCED_SHADER = () -> rectSolidColorInstancedShader;
	public static final Supplier<ShaderProg2D> RECT_SOLID_TEX_COLOR_INSTANCED_SHADER = () -> rectSolidTexColorInstancedShader;
	public static final Supplier<ShaderProg2D> TEXT_INSTANCED_SHADER = () -> textInstancedShader;
	public static final Supplier<ShaderProg2D> GENERIC_INSTANCED_SHADER = () -> genericInstancedShader;

	private static final List<ShaderProg> shaders = OpenArrayList.wrap(new ShaderProg[0]);

	public static void reload() {
		try {
			shaders.clear();
			loader.clearCache();

			rectSolidColorInstancedShader = ProgramLoader.loadProgram(loader, "rect_instanced", "rect", ShaderProg2D::new);
			rectSolidTexColorInstancedShader = ProgramLoader.loadProgram(loader, "rect_tex_instanced", "rect_tex", ShaderProg2D::new);
			textInstancedShader = ProgramLoader.loadProgram(loader, "text_instanced", "text", ShaderProg2D::new);
			genericInstancedShader = ProgramLoader.loadProgram(loader, "generic_instanced", "generic_instanced", ShaderProg2D::new);

			shaders.add(rectSolidColorInstancedShader);
			shaders.add(rectSolidTexColorInstancedShader);
			shaders.add(textInstancedShader);
			shaders.add(genericInstancedShader);

//				GL.glUniform1i(((AccessorShader) rectSolidTexColorInstancedShader).samplerLocations().getFirst(), 0);
//				GL.glUniform1i(((AccessorShader) textInstancedShader).samplerLocations().getFirst(), 0);
//				GL.glUniform1i(((AccessorShader) genericInstancedShader).samplerLocations().getFirst(), 0);

			initialized.set(true);
		} catch (Throwable e) {
			LOGGER.log(() -> "Failed to load shaders", e, List.of(ERROR));
			throw throwUnchecked(e);
		}
	}

	public static void updateUniforms(float xScaleFactor, float yScaleFactor) {
		shaders.forEach(shader -> {
			if (shader instanceof ShaderProg2D shader2D) {
				if (!GL_FEATURES.canUseGlProgramUniformFunctions()) {
					GuiGlobals.GL.glUseProgram(shader2D.handle());
				}

				shader2D.windowScaleFactor.set(xScaleFactor, yScaleFactor);
			}
		});
	}
}
