package dev.pfaff.altargui.ffi;

import java.util.List;
import java.util.stream.Stream;

public record Expression(double constant, List<Term> terms) implements ToExpression {
	public static final Expression ZERO = new Expression(0.0);

	public Expression {
		// ensure we always have a trusted List implementation
		terms = List.copyOf(terms);
	}

	public Expression(double constant) {
		this(constant, List.of());
	}

	public Expression(Term... terms) {
		this(0.0, List.of(terms));
	}

	@Override
	public Expression toExpression() {
		return this;
	}

	public Expression plus(double constant) {
		return new Expression(this.constant + constant, this.terms);
	}

	@Override
	public Expression plus(ToExpression rhs) {
		return (Expression) ToExpression.super.plus(rhs);
	}

	@Override
	public Expression minus(ToExpression rhs) {
		return (Expression) ToExpression.super.minus(rhs);
	}

	@Override
	public Expression plus(Query rhs) {
		return (Expression) ToExpression.super.plus(rhs);
	}

	@Override
	public Expression minus(Query rhs) {
		return (Expression) ToExpression.super.minus(rhs);
	}

	@Override
	public Expression plus(Term rhs) {
		return new Expression(this.constant, Stream.concat(this.terms.stream(), Stream.of(rhs)).toList());
	}

	@Override
	public Expression minus(Term rhs) {
		return plus(rhs.negate());
	}

	public Expression plus(Expression rhs) {
		return new Expression(this.constant + rhs.constant,
				Stream.concat(this.terms.stream(), rhs.terms.stream()).toList());
	}

	@Override
	public Expression minus(Expression rhs) {
		return new Expression(this.constant - rhs.constant,
				Stream.concat(this.terms.stream(), rhs.terms.stream().map(Term::negate)).toList());
	}

	@Override
	public Expression times(double rhs) {
		return new Expression(this.constant * rhs, this.terms.stream().map(term -> term.times(rhs)).toList());
	}

	@Override
	public Expression dividedBy(double rhs) {
		return (Expression) ToExpression.super.dividedBy(rhs);
	}

	public Expression negate() {
		return new Expression(-this.constant, this.terms.stream().map(Term::negate).toList());
	}

	@Override
	public String toString() {
		var sb = new StringBuilder();
		sb.append(this.constant);
		for (var term : this.terms) {
			sb.append(" + ");
			sb.append(term);
		}
		return sb.toString();
	}
}
