package dev.pfaff.altargui.ffi;

import org.jetbrains.annotations.Nullable;

import java.lang.foreign.*;
import java.lang.invoke.MethodHandle;
import java.lang.invoke.VarHandle;
import java.nio.file.Path;
import java.util.Set;

import static java.lang.foreign.MemoryLayout.PathElement.groupElement;

final class FFI {
	private static final Linker LINKER = Linker.nativeLinker();
	private static final SymbolLookup LOOKUP = SymbolLookup.libraryLookup(Path.of("natives/altar-gui.dylib"),
			Arena.ofAuto());

	private static StructLayout ML_slice(MemoryLayout pointee) {
		return MemoryLayout.structLayout(ValueLayout.ADDRESS.withTargetLayout(pointee).withName("base"),
						ValueLayout.JAVA_LONG.withName("len"))
				.withName("&[" + pointee.name() + "]");
	}

	public static final ValueLayout.OfLong ML_Variable = ValueLayout.JAVA_LONG.withName("Variable");
	public static final ValueLayout.OfLong ML_ConstraintId = ValueLayout.JAVA_LONG.withName("ConstraintId");
	public static final AddressLayout ML_LayoutEngine_ptr = ValueLayout.ADDRESS
			.withName("*LayoutEngine");
	//	public static final StructLayout ML_LayoutContext = MemoryLayout.structLayout(ML_LayoutEngine_ptr)
	public static final AddressLayout ML_LayoutContext = ML_LayoutEngine_ptr
			.withName("LayoutContext");
	//	public static final StructLayout ML_LaidContext = MemoryLayout.structLayout(ML_LayoutEngine_ptr)
	public static final AddressLayout ML_LaidContext = ML_LayoutEngine_ptr
			.withName("LaidContext");
	public static final StructLayout ML_VarViewport = MemoryLayout.structLayout(ML_Variable.withName("x"),
					ML_Variable.withName("y"),
					ML_Variable.withName("width"),
					ML_Variable.withName("height"))
			.withName("Viewport<Variable>");
	public static final ValueLayout.OfByte ML_RelationalOperator = ValueLayout.JAVA_BYTE
			.withName("RelationalOperator");
	public static final StructLayout ML_Term = MemoryLayout.structLayout(
					ML_Variable.withName("variable"),
					ValueLayout.JAVA_DOUBLE.withName("coefficient")
			)
			.withName("Term");
	public static final StructLayout ML_Expression = MemoryLayout.structLayout(
					ValueLayout.JAVA_DOUBLE.withName("constant"),
					ML_slice(ML_Term).withName("terms")
			)
			.withName("Expression");
	public static final ValueLayout.OfDouble ML_Strength = ValueLayout.JAVA_DOUBLE;
	public static final StructLayout ML_Constraint = MemoryLayout.structLayout(
					ML_Expression.withName("expr"),
					ML_Strength.withName("strength"),
					ML_RelationalOperator.withName("op"),
					MemoryLayout.paddingLayout(Long.BYTES - ML_RelationalOperator.byteSize())
			)
			.withName("Constraint");

	public static final VarHandle VH_VarViewport_x =
			ML_VarViewport.varHandle(groupElement("x")).withInvokeExactBehavior();
	public static final VarHandle VH_VarViewport_y =
			ML_VarViewport.varHandle(groupElement("y")).withInvokeExactBehavior();
	public static final VarHandle VH_VarViewport_width =
			ML_VarViewport.varHandle(groupElement("width")).withInvokeExactBehavior();
	public static final VarHandle VH_VarViewport_height =
			ML_VarViewport.varHandle(groupElement("height")).withInvokeExactBehavior();

	public static final VarHandle VH_Constraint_expr_constant =
			ML_Constraint.varHandle(groupElement("expr"), groupElement("constant")).withInvokeExactBehavior();
	public static final VarHandle VH_Constraint_expr_terms_base =
			ML_Constraint.varHandle(groupElement("expr"),
					groupElement("terms"),
					groupElement("base")).withInvokeExactBehavior();
	public static final VarHandle VH_Constraint_expr_terms_len =
			ML_Constraint.varHandle(groupElement("expr"),
					groupElement("terms"),
					groupElement("len")).withInvokeExactBehavior();
	public static final VarHandle VH_Constraint_strength =
			ML_Constraint.varHandle(groupElement("strength")).withInvokeExactBehavior();
	public static final VarHandle VH_Constraint_op =
			ML_Constraint.varHandle(groupElement("op")).withInvokeExactBehavior();

	public static final VarHandle VH_Term_variable =
			ML_Term.varHandle(groupElement("variable")).withInvokeExactBehavior();
	public static final VarHandle VH_Term_coefficient =
			ML_Term.varHandle(groupElement("coefficient")).withInvokeExactBehavior();

	public static final MethodHandle LayoutEngine_nAllocate;
	public static final MethodHandle LayoutEngine_nDeallocate;
	public static final MethodHandle LayoutEngine_nGetAssertions;
	public static final MethodHandle LayoutEngine_nSetAssertions;

	public static final MethodHandle LayoutContext_nNewVariable;
	public static final MethodHandle LayoutContext_nAddConstraint;
	public static final MethodHandle LayoutContext_nRemoveConstraint;

	public static final MethodHandle LaidContext_nGetValue;
	public static final MethodHandle LaidContext_nDump;

	private static MethodHandle downcallHandle(String name, Set<Linker.Option> options,
											   @Nullable MemoryLayout retLayout, MemoryLayout... argLayouts) {
		return LINKER.downcallHandle(LOOKUP.find(name).get(),
				retLayout == null
						? FunctionDescriptor.ofVoid(argLayouts)
						: FunctionDescriptor.of(retLayout, argLayouts),
				options.toArray(Linker.Option[]::new));
	}

	static {
		LayoutEngine_nAllocate = downcallHandle("altargui_LayoutEngine_allocate",
				Set.of(),
				// ret
				ML_LayoutEngine_ptr);
		LayoutEngine_nDeallocate = downcallHandle("altargui_LayoutEngine_deallocate",
				Set.of(),
				// ret
				null,
				// args
				ML_LayoutEngine_ptr);
		LayoutEngine_nGetAssertions = downcallHandle("altargui_LayoutEngine_get_assertions",
				Set.of(Linker.Option.critical(false)),
				// ret
				ValueLayout.JAVA_BOOLEAN,
				// args
				ML_LayoutEngine_ptr);
		LayoutEngine_nSetAssertions = downcallHandle("altargui_LayoutEngine_set_assertions",
				Set.of(Linker.Option.critical(false)),
				// ret
				null,
				// args
				ML_LayoutEngine_ptr,
				ValueLayout.JAVA_BOOLEAN);

		LayoutContext_nNewVariable = downcallHandle("altargui_LayoutContext_new_variable",
				Set.of(Linker.Option.critical(false)),
				// ret
				ML_Variable,
				// args
				ML_LayoutContext);
		LayoutContext_nAddConstraint = downcallHandle(
				"altargui_LayoutContext_add_constraint",
				Set.of(),
				// ret
				ML_ConstraintId,
				// args
				ML_LayoutContext,
				ValueLayout.ADDRESS.withTargetLayout(ML_Constraint).withName("constraint"));
		LayoutContext_nRemoveConstraint = downcallHandle("altargui_LayoutContext_remove_constraint",
				Set.of(),
				// ret
				ValueLayout.JAVA_BOOLEAN,
				// args
				ML_LayoutContext,
				ML_ConstraintId);

		LaidContext_nGetValue = downcallHandle("altargui_LaidContext_get_value",
				Set.of(),
				// ret
				ValueLayout.JAVA_DOUBLE,
				// args
				ML_LaidContext,
				ML_Variable);
		LaidContext_nDump = downcallHandle("altargui_LaidContext_dump",
				Set.of(),
				// ret
				null,
				// args
				ML_LaidContext);
	}
}
