package dev.pfaff.altargui.ffi;

import java.lang.foreign.*;
import java.lang.invoke.MethodHandle;

import static dev.pfaff.altargui.ffi.FFI.*;
import static java.lang.foreign.MemoryLayout.PathElement.groupElement;
import static java.lang.foreign.MemoryLayout.PathElement.sequenceElement;

public final class FFILayoutContext {
	public final FFILayoutEngine engine;

	FFILayoutContext(FFILayoutEngine engine) {
		this.engine = engine;
	}


	/**
	 * @return the new variable reference, or {@code null} if the allocation failed.
	 */
	public Variable newVariable() {
		return this.engine.newVariable();
	}

	private static StructLayout ML_Constraint_with_Terms(long termCount) {
		var ML_terms = MemoryLayout.sequenceLayout(
				(Long.MAX_VALUE - ML_Constraint.byteSize()) / ML_Term.byteSize(),
				ML_Term);
		if (termCount != -1L) ML_terms = ML_terms.withElementCount(termCount);
		return MemoryLayout.structLayout(ML_Constraint, ML_terms);
	}

	private static final StructLayout ML_Constraint_with_Terms_template = ML_Constraint_with_Terms(-1L);
	private static final MethodHandle MH_sliceTerm = ML_Constraint_with_Terms_template.sliceHandle(
			groupElement(1),
			sequenceElement());

	private static void setVarViewport(MemorySegment segment, VarRect viewport) {
		VH_VarViewport_x.set(segment, viewport.x().raw());
		VH_VarViewport_y.set(segment, viewport.y().raw());
		VH_VarViewport_width.set(segment, viewport.width().raw());
		VH_VarViewport_height.set(segment, viewport.height().raw());
	}

	public ConstraintId addConstraint(Constraint constraint) {
		// get the address eagerly so we get the thread-safety check
		var address = engine.address();

		try (var arena = Arena.ofConfined()) {
			var terms = constraint.expr().terms();
			var layout = ML_Constraint_with_Terms(terms.size());
			MemorySegment fullSegment = arena.allocate(layout);
			MemorySegment constraintSegment = fullSegment.asSlice(0, ML_Constraint);
			VH_Constraint_expr_constant.set(constraintSegment, constraint.expr().constant());
			VH_Constraint_expr_terms_base.set(constraintSegment,
					constraintSegment.asSlice(layout.byteOffset(groupElement(1)), 0));
			VH_Constraint_expr_terms_len.set(constraintSegment, (long) terms.size());
			for (int i = 0; i < terms.size(); i++) {
				var term = terms.get(i);
				MemorySegment termSegment;
				try {
					termSegment = (MemorySegment) MH_sliceTerm.invokeExact(fullSegment, (long) i);
				} catch (Throwable e) {
					throw new RuntimeException(e);
				}
				VH_Term_variable.set(termSegment, (switch (term.query()) {
					case Variable v -> v;
					case ViewportQuery ignored ->
							throw new UnsupportedOperationException("TODO: refactor and implement queries elsewhere");
				}).raw());
				VH_Term_coefficient.set(termSegment, term.coefficient());
			}
			VH_Constraint_strength.set(constraintSegment, constraint.strength());
			VH_Constraint_op.set(constraintSegment, (byte) constraint.op().ordinal());

			long id;
			try {
				id = (long) LayoutContext_nAddConstraint.invokeExact(address, constraintSegment);
			} catch (Throwable e) {
				throw new RuntimeException(e);
			}
			return id == 0 ? null : new ConstraintIdImpl(id);
		}
	}

	public boolean removeConstraint(ConstraintId id) {
		try {
			return (boolean) LayoutContext_nRemoveConstraint.invokeExact(engine.address(), id.raw());
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}
}
