package dev.pfaff.altargui.ffi;

import java.lang.foreign.MemorySegment;

import static dev.pfaff.altargui.ffi.FFI.*;

public final class FFILayoutEngine implements AutoCloseable {
	private final Thread thread;
	private final MemorySegment address;
	private boolean closed;

	public FFILayoutEngine() {
		this.thread = Thread.currentThread();
		try {
			this.address = (MemorySegment) LayoutEngine_nAllocate.invokeExact();
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
		if (this.address == null) throw new RuntimeException("Allocation of native LayoutEngine failed");
	}

	MemorySegment address() {
		if (this.closed) throw new IllegalStateException("Closed");
		assert Thread.currentThread() == this.thread
				: "LayoutEngine is not thread-safe, but an attempt was made to use it from another thread!";
		return this.address;
	}

	@Override
	public void close() {
		if (this.closed) throw new IllegalStateException("Already closed");
		this.closed = true;
		try {
			LayoutEngine_nDeallocate.invokeExact(address());
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * @return the new variable reference, or {@code null} if the allocation failed.
	 */
	public Variable newVariable() {
		long id;
		try {
			id = (long) LayoutContext_nNewVariable.invokeExact(this.address());
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
		return id == 0 ? null : new VariableImpl(id);
	}

	public FFILayoutContext newLayoutContext() {
		return new FFILayoutContext(this);
	}

	public FFILaidOutContext newLaidOutContext() {
		return new FFILaidOutContext(this);
	}
}
