package dev.pfaff.altargui.ffi;

public sealed interface ToExpression permits Expression, Query, Term {
	Expression toExpression();

	default Expression plus(double rhs) {
		return toExpression().plus(rhs);
	}

	default Expression minus(double constant) {
		return plus(-constant);
	}

	default ToExpression plus(ToExpression rhs) {
		return switch (rhs) {
			case Query q -> plus(q);
			case Term t -> plus(t);
			case Expression e -> plus(e);
		};
	}

	default ToExpression minus(ToExpression rhs) {
		return switch (rhs) {
			case Query q -> minus(q);
			case Term t -> minus(t);
			case Expression e -> minus(e);
		};
	}

	default ToExpression plus(Query rhs) {
		return plus(new Term(rhs));
	}

	default ToExpression minus(Query rhs) {
		return plus(rhs.negate());
	}

	default ToExpression plus(Term rhs) {
		return toExpression().plus(rhs);
	}

	default ToExpression minus(Term rhs) {
		return toExpression().minus(rhs);
	}

	default ToExpression plus(Expression rhs) {
		return toExpression().plus(rhs);
	}

	default ToExpression minus(Expression rhs) {
		return toExpression().minus(rhs);
	}

	default ToExpression times(double rhs) {
		return toExpression().times(rhs);
	}

	default ToExpression dividedBy(double rhs) {
		return times(1.0 / rhs);
	}

	ToExpression negate();

	default Constraint ge(ToExpression rhs) {
		return new Constraint(this.minus(rhs).toExpression(), Strength.REQUIRED, RelationalOperator.GreaterOrEqual);
	}

	default Constraint eq(ToExpression rhs) {
		return new Constraint(this.minus(rhs).toExpression(), Strength.REQUIRED, RelationalOperator.Equal);
	}

	default Constraint le(ToExpression rhs) {
		return new Constraint(this.minus(rhs).toExpression(), Strength.REQUIRED, RelationalOperator.LessOrEqual);
	}

	default Constraint ge(double rhs) {
		return ge(new Expression(rhs));
	}

	default Constraint eq(double rhs) {
		return eq(new Expression(rhs));
	}

	default Constraint le(double rhs) {
		return le(new Expression(rhs));
	}
}
