package dev.pfaff.altargui.function;

import dev.pfaff.altargui.inspector.ValueRenderers;

@FunctionalInterface
public interface ByteProvider {
	void provide(ByteConsumer consumer);

	static ByteProvider ofNone() {
		return None.INSTANCE;
	}

	static ByteProvider of(byte value) {
		return new Single(value);
	}

	static ByteProvider of(String string) {
		return new OfString(string);
	}

	static final class None implements ByteProvider {
		private static final None INSTANCE = new None();

		private None() {}

		@Override
		public void provide(ByteConsumer consumer) {
		}
	}

	record Single(byte value) implements ByteProvider {
		static {
			ValueRenderers.registerExactRendererBy(Single.class, ValueRenderers.Kind.Preview, Single::value);
		}

		@Override
		public void provide(ByteConsumer consumer) {
			consumer.accept(value);
		}
	}

	record OfString(String string) implements ByteProvider {
		static {
			ValueRenderers.registerExactRendererBy(OfString.class, ValueRenderers.Kind.Preview, OfString::string);
		}

		@Override
		public void provide(ByteConsumer consumer) {
			string.chars().forEach(consumer);
		}
	}
}
