package dev.pfaff.altargui.function;

import dev.pfaff.altargui.inspector.ValueRenderers;

public interface CharProvider {
	int length();

	void provide(CharConsumer consumer);

	static CharProvider ofNone() {
		return None.INSTANCE;
	}

	static CharProvider of(char value) {
		return new Single(value);
	}

	static CharProvider of(String string) {
		return new OfString(string);
	}

	static final class None implements CharProvider {
		private static final None INSTANCE = new None();

		private None() {}

		@Override
		public int length() {
			return 0;
		}

		@Override
		public void provide(CharConsumer consumer) {
		}
	}

	record Single(char value) implements CharProvider {
		static {
			ValueRenderers.registerExactRendererBy(Single.class, ValueRenderers.Kind.Preview, Single::value);
		}

		@Override
		public int length() {
			return 1;
		}

		@Override
		public void provide(CharConsumer consumer) {
			consumer.accept(value);
		}
	}

	record OfString(String string) implements CharProvider {
		static {
			ValueRenderers.registerExactRendererBy(OfString.class, ValueRenderers.Kind.Preview, OfString::string);
		}

		@Override
		public int length() {
			return string.length();
		}

		@Override
		public void provide(CharConsumer consumer) {
			string.chars().forEach(consumer);
		}
	}
}
