package dev.pfaff.altargui.host;

import dev.pfaff.altargui.debug.Profiler;
import dev.pfaff.altargui.text.Font;
import dev.pfaff.altargui.text.MinecraftBitmapFont;
import dev.pfaff.glass.opengl.GL;
import org.jspecify.annotations.Nullable;

import java.lang.foreign.SymbolLookup;

/**
 *
 */
public interface Host {
	/**
	 * @implSpec The return value must be constant across invocations per instance.
	 */
	default Font defaultFont() {
		return MinecraftBitmapFont.MINECRAFT_ASCII;
	}

	/**
	 * @implSpec The return value must be constant across invocations per instance.
	 */
	GL gl();

	/**
	 * @implSpec The return value must be constant across invocations per instance.
	 */
	IWindow window();

	@Nullable
	String getClipboardString();

	/**
	 * @param string the string. This should not contain NUL bytes. If it does, it will be truncated at those bytes.
	 */
	void setClipboardString(String string);

	SymbolLookup loadLibImage();

	/**
	 * @return the context forker, if supported by the host, or {@code null} otherwise.
	 * @implSpec The return value must be constant across invocations per instance.
	 */
	@Nullable
	default ContextForker contextForker() {
		return null;
	}

	default Profiler profiler() {
		return Profiler.NOP;
	}
}
