package dev.pfaff.altargui.improv;

import dev.pfaff.altargui.ffi.Constraint;
import dev.pfaff.altargui.ffi.ConstraintId;
import it.unimi.dsi.fastutil.longs.LongArrayList;
import org.jetbrains.annotations.Nullable;

import static dev.pfaff.altargui.util.ThrowUtil.throwUnchecked;

public final class ConstraintGroup {
	private final Element element;
	private final LayoutApplier applier;

	private final LongArrayList constraints = new LongArrayList();
	private State state = State.Reset;

	private enum State {
		/**
		 * Update queued.
		 */
		QueuedUpdate,
		/**
		 * Reset, possibly queued.
		 */
		Reset,
		/**
		 * Updated. Not queued.
		 */
		Updated;
	}

	public ConstraintGroup(Element element, LayoutApplier applier) {
		this.element = element;
		this.applier = applier;
	}

	public void queueUpdate() {
		if (this.state == State.QueuedUpdate) return;
		this.state = State.QueuedUpdate;
		this.element.queueLayout(this::updateImmediately);
	}

	public void queueReset() {
		if (this.state == State.Reset) return;
		this.state = State.Reset;
		this.element.queueLayout(this::resetImmediately);
	}

	public void updateImmediately(LayoutContext context) throws LayoutException {
		this.resetImmediately(context);
		this.state = State.Updated;
		this.applier.apply(new ApplyLayoutContextImpl(context));
	}

	@SuppressWarnings("RedundantThrows")
	public void resetImmediately(LayoutContext context) throws LayoutException {
		this.state = State.Reset;
		this.constraints.forEach(id -> {
			try {
				context.removeConstraint(ConstraintId.fromRaw(id));
			} catch (LayoutException e) {
				throw throwUnchecked(e);
			}
		});
		this.constraints.clear();
	}

	private final class ApplyLayoutContextImpl implements ApplyLayoutContext {
		private final LayoutContext context;

		private ApplyLayoutContextImpl(LayoutContext context) {
			this.context = context;
		}

		@Override
		public LayoutContext context() {
			return this.context;
		}

		@Override
		public void addConstraint(@Nullable String comment, Constraint constraint) throws LayoutException {
			ConstraintGroup.this.constraints.add(this.context.addConstraint(comment, constraint).raw());
		}
	}
}
