package dev.pfaff.altargui.improv;

import dev.pfaff.altargui.ffi.FFILayoutEngine;
import dev.pfaff.altargui.ffi.VarRect;
import dev.pfaff.altargui.ffi.Variable;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public interface Context {
	static Context create() {
		return new ContextImpl();
	}

	/**
	 * Checks that the current thread is the context's UI thread.
	 */
	void checkThread();

	FFILayoutEngine ffiEngine();

	default Variable newVariable() {
		return ffiEngine().newVariable();
	}

	default VarRect newVarRect() {
		return VarRect.create(ffiEngine());
	}

	/**
	 * Submits an event to the UI thread for "immediate" execution.
	 */
	void submitImmediateEvent(Runnable event);

	void processEvents();

	void mountChildImmediately(@Nullable Element parent, @NotNull Element child);

	void unmountChildImmediately(@Nullable Element parent, @NotNull Element child);

	/**
	 * Mounts the child with the given parent.
	 * @param parent
	 * @param child
	 */
	default void mountChild(@Nullable Element parent, @NotNull Element child) {
		submitImmediateEvent(() -> this.mountChildImmediately(parent, child));
	}

	default void unmountChild(@Nullable Element parent, @NotNull Element child) {
		submitImmediateEvent(() -> this.unmountChildImmediately(parent, child));
	}
}
