package dev.pfaff.altargui.improv;

import dev.pfaff.altargui.ffi.FFILayoutEngine;
import it.unimi.dsi.fastutil.objects.ObjectArrayList;

import java.io.Closeable;
import java.util.List;
import java.util.Objects;

import static dev.pfaff.altargui.util.ThrowUtil.throwUnchecked;

public final class ContextImpl implements Context, Closeable {
	// ObjectArrayList is intentionally chosen over OpenArrayList because its forEach implementation supports
	// appending during traversal (and includes appended elements in said traversal).
	private final List<Runnable> immediateEvents = ObjectArrayList.wrap(new Runnable[0]);
	private final FFILayoutEngine ffiLayoutEngine = new FFILayoutEngine();
	private final Thread thread = Thread.currentThread();

	@Override
	public void checkThread() {
		if (Thread.currentThread() != this.thread) {
			throw new IllegalStateException("Not on UI thread: expected " + this.thread + ", found " + Thread.currentThread());
		}
	}

	@Override
	public FFILayoutEngine ffiEngine() {
		return ffiLayoutEngine;
	}

	@Override
	public void submitImmediateEvent(Runnable event) {
		immediateEvents.add(event);
	}

	@Override
	public void processEvents() {
		immediateEvents.forEach(Runnable::run);
		immediateEvents.clear();
	}

	@Override
	public void mountChildImmediately(Element parent, Element child) {
		Objects.requireNonNull(child, "Argument 'child' must be non-null");
		child.onMount(parent);
	}

	@Override
	public void unmountChildImmediately(Element parent, Element child) {
		Objects.requireNonNull(child, "Argument 'child' must be non-null");
		assert parent == child.parent() : "Attempted to unmount a child from a widget that is not its parent";
		child.onUnmount();
	}

	@Override
	public void close() {
		ffiLayoutEngine.close();
	}
}
