package dev.pfaff.altargui.improv;

import dev.pfaff.altargui.ffi.*;
import org.jetbrains.annotations.Nullable;

import java.util.Arrays;
import java.util.List;

public interface LayoutContext {
	Context context();

	FFILayoutContext ffiLayoutContext();

	default Variable newVariable() {
		return context().newVariable();
	}

	default VarRect newVarRect() {
		return context().newVarRect();
	}

	/**
	 * Adds a new <strong>untracked</strong> constraint to the system.
	 */
	ConstraintId addConstraint(@Nullable String comment, Constraint constraint) throws LayoutException;
	void removeConstraint(ConstraintId id) throws LayoutException;

	default void addConstraints(@Nullable String comment, Constraint... constraints) throws LayoutException {
		Util.dumpConstraints(comment, constraints);
		for (int i = 0; i < constraints.length; i++) {
			var constraint = constraints[i];
			try {
				this.addConstraint(null, constraint);
			} catch (AddConstraintException e) {
				throw new AddConstraintException(comment, e.index, e.constraint, e.getCause());
			} catch (Throwable e) {
				throw new AddConstraintException(comment, i, constraint, e);
			}
		}
	}

	/**
	 * The space made available to the element by the parent.
	 */
	@Nullable ExprRect available();

	default boolean assertions() {
		return false;
	}
}
