package dev.pfaff.altargui.improv.elements;

import dev.pfaff.altargui.ffi.ExprRect;
import dev.pfaff.altargui.ffi.Expression;
import dev.pfaff.altargui.improv.*;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;

/**
 * Absolute positioned element. Exists outside the regular layout.
 */
public final class AbsoluteElement extends AbstractSingleChildElement {
	private ExprRect absAvailable;

	private final ConstraintGroup childXGroup = new ConstraintGroup(this, context -> {
		var child = this.getChild();
		if (child == null) return;
		var childAllocation = child.allocation();
		context.addConstraint("Absolute[child::x]", childAllocation.x().eq(this.absAvailable.x()));
	});

	private final ConstraintGroup childYGroup = new ConstraintGroup(this, context -> {
		var child = this.getChild();
		if (child == null) return;
		var childAllocation = child.allocation();
		context.addConstraint("Absolute[child::y]", childAllocation.y().eq(this.absAvailable.y()));
	});

	public AbsoluteElement(Context context) {
		super(context);
		this.absAvailable = new ExprRect(Expression.ZERO, Expression.ZERO, context.newVariable(), context.newVariable());
	}

	public void setX(@NotNull Expression expr) {
		Objects.requireNonNull(expr);
		if (!expr.equals(this.absAvailable.x())) {
			this.absAvailable = this.absAvailable.withX(expr);
			var child = this.getChild();
			if (child != null) child.setAvailable(this.absAvailable);
			this.childXGroup.queueUpdate();
		}
	}

	public void setY(@NotNull Expression expr) {
		Objects.requireNonNull(expr);
		if (!expr.equals(this.absAvailable.y())) {
			this.absAvailable = this.absAvailable.withY(expr);
			var child = this.getChild();
			if (child != null) child.setAvailable(this.absAvailable);
			this.childYGroup.queueUpdate();
		}
	}

	@Override
	protected void onMountChild(Element child) {
		super.onMountChild(child);
		child.setAvailable(this.absAvailable);
		this.childXGroup.queueUpdate();
		this.childYGroup.queueUpdate();
	}

	@Override
	protected void onUnmountChild(Element child) {
		this.childXGroup.queueReset();
		this.childYGroup.queueReset();
		super.onUnmountChild(child);
	}
}
