package dev.pfaff.altargui.improv.elements;

import dev.pfaff.altargui.ffi.ExprRect;
import dev.pfaff.altargui.improv.Context;
import dev.pfaff.altargui.improv.Element;
import org.jetbrains.annotations.Nullable;

public abstract class AbstractSingleChildElement extends Element {
	private Element child;

	protected final ExprRect childAvailable;

	public AbstractSingleChildElement(Context context) {
		super(context);
		this.childAvailable = new ExprRect(context.newVarRect());
	}

	public void setChild(Element child) {
		if (child != this.child) {
			var oldChild = this.child;
			this.context().submitImmediateEvent(() -> {
				if (oldChild != null) {
					this.onUnmountChild(child);
					this.context().unmountChildImmediately(this, oldChild);
				}
				if (child != null) {
					this.context().mountChildImmediately(this, child);
					this.onMountChild(child);
				}
			});
			this.child = child;
		}
	}

	public @Nullable Element getChild() {
		return this.child;
	}

	protected void onMountChild(Element child) {}

	protected void onUnmountChild(Element child) {}
}
