package dev.pfaff.altargui.improv.elements;

import dev.pfaff.altargui.ffi.Expression;
import dev.pfaff.altargui.improv.*;
import dev.pfaff.altargui.types.Alignment;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;

public final class PaddingElement extends AbstractSingleChildElement {
	private Expression startX = Expression.ZERO;
	private Expression startY = Expression.ZERO;
	private Expression endX = Expression.ZERO;
	private Expression endY = Expression.ZERO;

	public PaddingElement(Context context) {
		super(context);
	}

	private final ConstraintGroup groupAvailable = new ConstraintGroup(this, context -> {
		var available = context.available();
		if (available == null) return;
		var allocation = this.allocation();
		System.out.println("Padding allocation=" + allocation + ", available=" + available);
		{
			var laidContext = context.context().ffiLayoutContext().engine.newLaidOutContext();
			System.out.println("Padding allocation=" + allocation.getValues(laidContext));
		}
		context.addConstraints(
				"Padding[self::align]",
				Constraints.align(allocation, available, Alignment.Start, Alignment.Start)
		);
		context.addConstraints(
				"Padding[self::fits]",
				Constraints.fits(allocation, available, false)
		);
	});
	private final ConstraintGroup groupPadding = new ConstraintGroup(this, context -> {
		var allocation = this.allocation();
		// require the padding element viewport to be exactly child + padding
		context.addConstraints(
				"Padding[self::child_plus_padding]",
				this.childAvailable.x().eq(allocation.x().plus(this.startX)),
				this.childAvailable.y().eq(allocation.y().plus(this.startY)),
				this.childAvailable.width().eq(allocation.width().minus(this.startX.plus(this.endX))),
				this.childAvailable.height().eq(allocation.height().minus(this.startY.plus(this.endY)))
		);
		if (context.assertions()) {
			// require the padding to be positive
			context.addConstraints(
					"Padding[assertions]",
					this.startX.ge(0.0),
					this.startY.ge(0.0),
					this.endX.ge(0.0),
					this.endY.ge(0.0)
			);
		}
	});

	public void setStartX(@NotNull Expression expr) {
		Objects.requireNonNull(expr);
		if (!expr.equals(this.startX)) {
			this.startX = expr;
			this.groupPadding.queueUpdate();
		}
	}

	public void setStartY(@NotNull Expression expr) {
		Objects.requireNonNull(expr);
		if (!expr.equals(this.startY)) {
			this.startY = expr;
			this.groupPadding.queueUpdate();
		}
	}

	public void setEndX(@NotNull Expression expr) {
		Objects.requireNonNull(expr);
		if (!expr.equals(this.endX)) {
			this.endX = expr;
			this.groupPadding.queueUpdate();
		}
	}

	public void setEndY(@NotNull Expression expr) {
		Objects.requireNonNull(expr);
		if (!expr.equals(this.endY)) {
			this.endY = expr;
			this.groupPadding.queueUpdate();
		}
	}

	@Override
	protected void onMountChild(Element child) {
		super.onMountChild(child);
		child.setAvailable(this.childAvailable);
	}

	@Override
	protected void onSetAvailable() {
		super.onSetAvailable();
		this.groupAvailable.queueUpdate();
	}
}
