package dev.pfaff.altargui.improv.elements;

import dev.pfaff.altargui.ffi.ConstraintId;
import dev.pfaff.altargui.ffi.ExprRect;
import dev.pfaff.altargui.ffi.Expression;
import dev.pfaff.altargui.improv.*;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;

/**
 * Absolute sized element.
 */
public final class SizedElement extends AbstractSingleChildElement {
	private Expression width = Expression.ZERO;
	private Expression height = Expression.ZERO;

	private ConstraintId constraintWidth;
	private ConstraintId constraintHeight;

	public SizedElement(Context context) {
		super(context);
	}

	private void updateConstraintWidth(LayoutContext context) throws LayoutException {
		if (this.constraintWidth != null) context.removeConstraint(this.constraintWidth);
		this.constraintWidth = context.addConstraint("Sized[self::width]", this.allocation().width().eq(this.width));
	}

	private void updateConstraintHeight(LayoutContext context) throws LayoutException {
		if (this.constraintHeight != null) context.removeConstraint(this.constraintHeight);
		this.constraintHeight = context.addConstraint("Sized[self::height]", this.allocation().height().eq(this.height));
	}

	public void setWidth(@NotNull Expression expr) {
		Objects.requireNonNull(expr);
		if (!expr.equals(this.width)) {
			this.width = expr;
			this.queueLayout(this::updateConstraintWidth);
		}
	}

	public void setHeight(@NotNull Expression expr) {
		Objects.requireNonNull(expr);
		if (!expr.equals(this.height)) {
			this.height = expr;
			this.queueLayout(this::updateConstraintHeight);
		}
	}

	@Override
	protected void onMountChild(Element child) {
		super.onMountChild(child);
		child.setAvailable(new ExprRect(this.allocation()));
	}
}
