package dev.pfaff.altargui.improv.elements;

import dev.pfaff.altargui.ffi.ExprRect;
import dev.pfaff.altargui.ffi.Strength;
import dev.pfaff.altargui.improv.ConstraintGroup;
import dev.pfaff.altargui.improv.Context;
import dev.pfaff.altargui.improv.Element;
import dev.pfaff.altargui.util.OpenArrayList;

public final class StackElement extends AbstractMultiChildElement {
	private final OpenArrayList<ConstraintGroup> childGroups = OpenArrayList.wrap(new ConstraintGroup[0]);

	public StackElement(Context context) {
		super(context);
	}

	private final ConstraintGroup groupAvailable = ExpandElement.makeAvailableGroup(this, "Stack");

	@Override
	protected void onMountChild(int index, Element child, boolean moved) {
		super.onMountChild(index, child, moved);
		if (moved) return;
		child.setAvailable(new ExprRect(this.allocation()));
		if (index >= this.childGroups.size()) {
			this.childGroups.add(new ConstraintGroup(this, context -> {
				var allocation = this.allocation();
				var childAllocation = this.getChild(index).allocation();
				context.addConstraints("Stack[child:pos]",
						childAllocation.x().eq(allocation.x()),
						childAllocation.y().eq(allocation.y()));
				context.addConstraints("Stack[self::weak_le_child]",
						allocation.width().eq(childAllocation.width()).withStrength(Strength.WEAK),
						allocation.height().eq(childAllocation.height()).withStrength(Strength.WEAK));
			}));
		}
		this.childGroups.get(index).queueUpdate();
	}

	@Override
	protected void onUnmountChild(int index, Element child) {
		super.onUnmountChild(index, child);
		this.childGroups.get(index).queueReset();
	}

	@Override
	protected void onSetAvailable() {
		super.onSetAvailable();
		this.groupAvailable.queueUpdate();
	}
}
