package dev.pfaff.altargui.improv.elements;

import dev.pfaff.altargui.function.CharConsumer;
import dev.pfaff.altargui.function.CharProvider;
import dev.pfaff.altargui.improv.ConstraintGroup;
import dev.pfaff.altargui.improv.Constraints;
import dev.pfaff.altargui.improv.Context;
import dev.pfaff.altargui.improv.Element;

import java.util.Objects;

public final class TextElement extends Element {
	private static final double LINE_HEIGHT = 8.0;

	private CharProvider text;

	public TextElement(Context context, CharProvider text) {
		super(context);
		this.text = text;
		this.groupSize.queueUpdate();
	}

	private final ConstraintGroup groupSize = new ConstraintGroup(this, context -> {
		double max_width, height, line_width;
		{
			var consumer = new CharConsumer() {
				double max_width = 0.0;
				double height = LINE_HEIGHT;
				double line_width = 0.0;

				@Override
				public void accept(char c) {
					if (c == '\n') {
						height += LINE_HEIGHT;
						max_width = Math.max(max_width, line_width);
						line_width = 0.0;
					} else {
						// FIXME: extent depends on character
						line_width += 8.0;
					}
				}
			};
			this.text.provide(consumer);
			max_width = consumer.max_width;
			height = consumer.height;
			line_width = consumer.line_width;
		}

		max_width = Math.max(max_width, line_width);
		var viewport = this.allocation();
		// Give the element the required size.
		context.addConstraints(
				"Text[self::requires_size]",
				Constraints.eqlSize(viewport, max_width, height)
		);
	});

	private final ConstraintGroup groupAvailable = ExpandElement.makeAvailableGroup(this, "Text");

	public void setText(CharProvider text) {
		Objects.requireNonNull(text);
		if (!text.equals(this.text)) {
			this.text = text;
			this.groupSize.queueUpdate();
		}
	}

	@Override
	protected void onSetAvailable() {
		super.onSetAvailable();
		groupAvailable.queueUpdate();
	}
}
