package dev.pfaff.altargui.input;

import dev.pfaff.glass.OS;

import java.util.List;

public enum InputMod {
	/** If this bit is set one or more Shift keys were held down. */
	SHIFT,

	/** If this bit is set one or more Control keys were held down. */
	CTRL,

	/** If this bit is set one or more Alt keys were held down. */
	ALT,

	/** If this bit is set one or more Super keys were held down. */
	SUPER,

	/** If this bit is set the Caps Lock key is enabled and the {@code LOCK_KEY_MODS} input mode is set. */
	CAPS_LOCK,

	/** If this bit is set the Num Lock key is enabled and the {@code LOCK_KEY_MODS} input mode is set. */
	NUM_LOCK,

	REPEAT;

	public static final List<InputMod> VALUES = List.of(values());

	/**
	 * {@link #CTRL} on all operating systems except for macOS, where it is {@link #SUPER}.
	 */
	public static final InputMod PRIMARY = switch (OS.HOST) {
		case OS.macOS _ -> InputMod.SUPER;
		default -> InputMod.CTRL;
	};

	public final int bit() {
		return 0b1 << this.ordinal();
	}

	public final boolean isAction() {
		return switch (this) {
			case SHIFT, CAPS_LOCK, NUM_LOCK, REPEAT -> false;
			case CTRL, ALT, SUPER -> true;
		};
	}
}
