package dev.pfaff.altargui.inspector;

import dev.pfaff.altargui.Inspector;
import dev.pfaff.altargui.Widget;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;
import java.util.function.Function;

/**
 * Handles building a widget to display the value.
 *
 * @param <T> the type that this renderer can handle.
 */
@FunctionalInterface
public interface ValueRenderer<T> {
    @NotNull Widget build(Inspector inspector, T value);

    static <T, R> ValueRenderer<T> by(Function<T, R> func, ValueRenderers.Kind kind) {
        return switch (kind) {
			case Preview -> (inspector, value) -> {
                var v = func.apply(value);
                return Objects.requireNonNull(ValueRenderers.forValue(v, ValueRenderers.Kind.Preview)).build(inspector, v);
            };
			case Detailed -> (inspector, value) -> {
                var v = func.apply(value);
                return Objects.requireNonNull(ValueRenderers.forValue(v, ValueRenderers.Kind.Detailed)).build(inspector, v);
            };
		};
    }

    @FunctionalInterface
    interface OfInt extends ValueRenderer<Integer> {
        @Override
        @Deprecated
        default @NotNull Widget build(Inspector inspector, Integer value) {
            return build(inspector, (int) value);
        }

        @NotNull Widget build(Inspector inspector, int value);
    }
}
