package dev.pfaff.altargui.inspector.renderers;

import dev.pfaff.altargui.Inspector;
import dev.pfaff.altargui.MinecraftColors;
import dev.pfaff.altargui.inspector.DebugPropertyValue;
import dev.pfaff.altargui.inspector.ValueRenderer;
import dev.pfaff.altargui.widgets.FrameBuilderWidget;
import dev.pfaff.altargui.widgets.TextWidget;

import java.lang.reflect.Modifier;
import java.util.Arrays;
import java.util.List;

import static dev.pfaff.altargui.Inspector.withInspectorFontSize;
import static dev.pfaff.altargui.widgets.MultiChildWidget.col;
import static dev.pfaff.altargui.widgets.MultiChildWidget.row;
import static dev.pfaff.altargui.widgets.TrackingWidgets.trackingIdentity;

public final class ClassValueRenderer {
	public static final ValueRenderer<Class> PREVIEW = (inspector, value) -> trackingIdentity(value, a -> {
		return withInspectorFontSize(TextWidget.of(a.getName()));
	});

	public static final ValueRenderer<Class> DETAILED = (inspector, value) -> trackingIdentity(value, a -> {
		return col(List.of(
				inspector.buildKeyValueEntry(withInspectorFontSize(TextWidget.of("Name")), DebugPropertyValue.of(a.getName()))
				, inspector.buildKeyValueEntry(withInspectorFontSize(TextWidget.of("Superclass")), DebugPropertyValue.of(a.getSuperclass()))
				, inspector.buildKeyValueEntry(withInspectorFontSize(TextWidget.of("Modifiers")), DebugPropertyValue.of(a.getModifiers()))
				, Inspector.labeledCol("Access flags", a.accessFlags()
						.stream()
						.map(field -> inspector.buildValuePreview(DebugPropertyValue.of(field)))
						.toList())
				// TODO: allow viewing static field values
				, Inspector.labeledCol("Fields", Arrays.stream(a.getDeclaredFields())
						.map(field -> {
							var name = inspector.buildValuePreview(DebugPropertyValue.of(new FieldNameRenderer(field)));
							if (Modifier.isStatic(field.getModifiers())) {
								if (field.canAccess(null)) {
									try {
										return row(List.of(name, Inspector.NAME_VALUE_SEPARATOR,
												Modifier.isFinal(field.getModifiers()) ? inspector.buildValuePreview(
														DebugPropertyValue.of(field.get(null))) : new FrameBuilderWidget(_ -> {
													try {
														return inspector.buildValuePreview(DebugPropertyValue.of(field.get(null)));
													} catch (IllegalAccessException e) {
														throw new RuntimeException(e);
													}
												})));
									} catch (IllegalAccessException e) {
										throw new RuntimeException(e);
									}
								} else {
									return row(List.of(name, Inspector.NAME_VALUE_SEPARATOR,
											withInspectorFontSize(TextWidget.of("Unable to access field"))
													.withColor(MinecraftColors.RED)));
								}
							} else {
								return name;
							}
						})
						.toList())
				, Inspector.labeledCol("Methods", Arrays.stream(a.getDeclaredMethods())
						.map(field -> inspector.buildValuePreview(DebugPropertyValue.of(field)))
						.toList())
		));
	});
}
