package dev.pfaff.altargui.inspector.renderers;

import dev.pfaff.altargui.Inspector;
import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.inspector.DebugPropertyValue;
import dev.pfaff.altargui.inspector.ValueRenderer;
import dev.pfaff.altargui.inspector.ValueRenderers;
import dev.pfaff.altargui.widgets.TextWidget;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.List;

import static dev.pfaff.altargui.Inspector.withInspectorFontSize;
import static dev.pfaff.altargui.widgets.MultiChildWidget.col;
import static dev.pfaff.altargui.widgets.TrackingWidgets.trackingIdentity;
import static java.lang.invoke.MethodType.methodType;

public final class FieldValueRenderer {

	public static final ValueRenderer<Field> PREVIEW = (inspector, value) -> trackingIdentity(value, a -> {
		return withInspectorFontSize(TextWidget.of(a.getType().getName() + " " + a.getName()));
	});

	public static final ValueRenderer<Field> DETAILED = (inspector, value) -> trackingIdentity(value, a -> {
		return col(List.of(
				inspector.buildKeyValueEntry(withInspectorFontSize(TextWidget.of("Name")),
						DebugPropertyValue.of(a.getName()))
				, inspector.buildKeyValueEntry(withInspectorFontSize(TextWidget.of("Type")),
						DebugPropertyValue.of(a.getType()))
				, inspector.buildKeyValueEntry(withInspectorFontSize(TextWidget.of("Modifiers")),
						DebugPropertyValue.of(a.getModifiers()))
				, Inspector.labeledCol("Access flags", a.accessFlags()
						.stream()
						.map(field -> inspector.buildValuePreview(DebugPropertyValue.of(field)))
						.toList())
		));
	});
}
