package dev.pfaff.altargui.inspector.renderers;

import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.inspector.ValueRenderer;
import dev.pfaff.altargui.widgets.TextWidget;

import java.lang.invoke.MethodType;
import java.util.HashMap;
import java.util.StringJoiner;

import static dev.pfaff.altargui.Inspector.withInspectorFontSize;

public final class MethodTypeValueRenderer {
    private static final HashMap<MethodType, String> METHOD_TYPE_STRING_CACHE = new HashMap<>();
    private static final HashMap<MethodType, Widget> METHOD_TYPE_WIDGET_CACHE = new HashMap<>();

    public static String format(MethodType value) {
        return METHOD_TYPE_STRING_CACHE.computeIfAbsent(value, signature -> {
            int nParams = signature.parameterCount();
            if (nParams == 1) {
                return "() -> " + signature.returnType().getSimpleName();
            } else {
                StringJoiner sj = new StringJoiner(", ", "(", ") -> " + signature.returnType().getSimpleName());
                for (int i = 0; i < nParams; i++) {
                    sj.add(signature.parameterType(i).getSimpleName());
                }
                return sj.toString();
            }
        });
    }

    public static final ValueRenderer<MethodType> PREVIEW = (inspector, value) -> {
        return METHOD_TYPE_WIDGET_CACHE.computeIfAbsent(value, v -> withInspectorFontSize(TextWidget.of(format(v))));
    };
}
