package dev.pfaff.altargui.inspector.renderers;

import dev.pfaff.altargui.Inspector;
import dev.pfaff.altargui.inspector.DebugPropertyValue;
import dev.pfaff.altargui.inspector.ValueRenderer;
import dev.pfaff.altargui.widgets.TextWidget;

import java.lang.invoke.MethodType;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import static dev.pfaff.altargui.Inspector.withInspectorFontSize;
import static dev.pfaff.altargui.widgets.MultiChildWidget.col;
import static dev.pfaff.altargui.widgets.TrackingWidgets.trackingIdentity;
import static java.lang.invoke.MethodType.methodType;

public final class MethodValueRenderer {
	public static final ValueRenderer<Method> PREVIEW = (inspector, value) -> trackingIdentity(value, a -> {
		var retType = a.getReturnType();
		var paramTypes = a.getParameterTypes();
		var sb = new StringBuilder(retType.getName().length() +
								   1 +
								   a.getName().length() +
								   1 +
								   Arrays.stream(paramTypes).mapToInt(clazz -> clazz.getName().length()).sum() +
								   Math.max(paramTypes.length - 1, 0) * 2 +
								   1)
				.append(retType.getName())
				.append(" ")
				.append(a.getName())
				.append("(");
		for (int i = 0; i < paramTypes.length;) {
			var clazz = paramTypes[i];
			assert sb.capacity() >= clazz.getName().length();
			sb.append(clazz.getName());
			if (++i < paramTypes.length) {
				assert sb.capacity() >= 2;
				sb.append(", ");
			}
		}
		sb.append(")");
		return withInspectorFontSize(TextWidget.of(sb.toString()));
	});

	public static final ValueRenderer<Method> DETAILED = (inspector, value) -> trackingIdentity(value, a -> {
		return col(List.of(
				inspector.buildKeyValueEntry(withInspectorFontSize(TextWidget.of("Name")),
						DebugPropertyValue.of(a.getName()))
				, inspector.buildKeyValueEntry(withInspectorFontSize(TextWidget.of("Type")),
						DebugPropertyValue.of(methodType(a.getReturnType(), a.getParameterTypes())))
				, inspector.buildKeyValueEntry(withInspectorFontSize(TextWidget.of("Modifiers")),
						DebugPropertyValue.of(a.getModifiers()))
				, Inspector.labeledCol("Access flags", a.accessFlags()
						.stream()
						.map(field -> inspector.buildValuePreview(DebugPropertyValue.of(field)))
						.toList())
		));
	});
}
