package dev.pfaff.altargui.internal;

import dev.pfaff.altargui.pathfinder.SceneProxyExecutor;
import dev.pfaff.altargui.render.*;

import java.util.function.Supplier;

import static dev.pfaff.glass.Config.*;

public class Config {
	public static final boolean CHECKS = getEnvFlag("ALTAR_GUI_CHECKS", false);
	public static final boolean SANITIZE_KEYS = getEnvFlag("ALTAR_GUI_SANITIZE_KEYS", false);
	/**
	 * When enabled, some unsafe APIs are used to perform accesses without bounds checks or to give the JIT compiler
	 * the necessary constant information to do so automatically.
	 */
	public static final boolean UNSAFE_OPTIMIZATIONS = getEnvFlag("ALTAR_GUI_UNSAFE_OPTIMIZATIONS", true);

	public static final boolean GL_CHECKS = getEnvFlag("ALTAR_GUI_GL_CHECKS", dev.pfaff.glass.opengl.GL.CHECKS);
	public static final boolean GL_ALWAYS_UNBIND = getEnvFlag("ALTAR_GUI_GL_ALWAYS_UNBIND", true);
	public static final int GL_CONTEXT_POOL_SIZE = getEnvInt("ALTAR_GUI_GL_CONTEXT_POOL_SIZE",
			// leave one core for the main thread.
			Math.max(Runtime.getRuntime().availableProcessors() - 1, 1));

	public static final Supplier<LayeredRenderBuffer> LAYERED_RENDER_BUFFER_IMPL = getEnvT(
			"ALTAR_GUI_LAYERED_RENDER_BUFFER_IMPL",
			GL31LayeredRenderBufferNew::new,
			s -> switch (s) {
				case "GL31LayeredRenderBuffer" -> GL31LayeredRenderBuffer::new;
				case "GL31LayeredRenderBufferNew" -> GL31LayeredRenderBufferNew::new;
				case "GL42DrawArraysIndirectLayeredRenderBuffer" -> GL42DrawArraysIndirectLayeredRenderBuffer::new;
				case "GL42DrawArraysInstancedBaseInstanceLayeredRenderBuffer" ->
						GL42DrawArraysInstancedBaseInstanceLayeredRenderBuffer::new;
				default -> throw new IllegalArgumentException("""
						Environment variable ALTAR_GUI_LAYERED_RENDER_BUFFER_IMPL must be set to one of:
						- GL31LayeredRenderBuffer
						- GL31LayeredRenderBufferNew
						- GL42DrawArraysIndirectLayeredRenderBuffer
						- GL42DrawArraysInstancedBaseInstanceLayeredRenderBuffer
						If unset, the highest performance default will be chosen for your system. This detection may not be perfect, however.""");
			});

	public static final String FONT_FAMILY = getEnvString("ALTAR_GUI_FONT_FAMILY", "SF Pro");
	public static final SceneProxyExecutor PATHFINDER_SCENE_PROXY_EXECUTOR = getEnvT("PATHFINDER_SCENE_PROXY_EXECUTOR",
			SceneProxyExecutor.Rayon,
			s -> switch (s) {
				case "Sequential" -> SceneProxyExecutor.Sequential;
				case "Rayon" -> SceneProxyExecutor.Rayon;
				default -> throw new IllegalArgumentException("""
						Environment variable PATHFINDER_SCENE_PROXY_EXECUTOR must be set to one of [Sequential, Rayon]. \
						If unset, it will default to Rayon.""");
			});
}
