package dev.pfaff.altargui.internal.render;

import dev.pfaff.unfettered.UnsafeWrapper;

import java.lang.foreign.Arena;
import java.lang.foreign.MemorySegment;
import java.lang.foreign.StructLayout;

import static java.lang.foreign.MemoryLayout.structLayout;
import static java.lang.foreign.ValueLayout.JAVA_FLOAT;

public final class ScissorStack {
	private static final StructLayout LAYOUT = structLayout(
			JAVA_FLOAT.withName("minX"),
			JAVA_FLOAT.withName("minY"),
			JAVA_FLOAT.withName("maxX"),
			JAVA_FLOAT.withName("maxY")
	).withByteAlignment(JAVA_FLOAT.byteAlignment() * 4);
	//private static final SequenceLayout SEQ_LAYOUT = sequenceLayout(LAYOUT);
//	private static final VarHandle VH_x = SEQ_LAYOUT.varHandle(sequenceElement(), groupElement("x"))
//			.withInvokeExactBehavior();
//	private static final VarHandle VH_y = SEQ_LAYOUT.varHandle(sequenceElement(), groupElement("y"))
//			.withInvokeExactBehavior();
//	private static final VarHandle VH_width = SEQ_LAYOUT.varHandle(sequenceElement(), groupElement("width"))
//			.withInvokeExactBehavior();
//	private static final VarHandle VH_height = SEQ_LAYOUT.varHandle(sequenceElement(), groupElement("height"))
//			.withInvokeExactBehavior();
//	private static final MethodHandle MH_xO = SEQ_LAYOUT.byteOffsetHandle(sequenceElement(), groupElement("x"));
//	private static final MethodHandle MH_yO = SEQ_LAYOUT.byteOffsetHandle(sequenceElement(), groupElement("y"));
//	private static final MethodHandle MH_widthO = SEQ_LAYOUT.byteOffsetHandle(sequenceElement(), groupElement("width"));
//	private static final MethodHandle MH_heightO = SEQ_LAYOUT.byteOffsetHandle(sequenceElement(), groupElement("height"));

	private MemorySegment buffer = Arena.ofAuto().allocate(LAYOUT, 64);
	private int length = 0;

	private int capacity() {
		return (int) this.buffer.byteSize() / (int) LAYOUT.byteSize();
	}

	private void grow() {
		int newCapacity = capacity() << 1;
		var newBuffer = Arena.ofAuto().allocate(LAYOUT, newCapacity);
		newBuffer.copyFrom(this.buffer);
		this.buffer = newBuffer;
	}

	public void push(float x, float y, float maxX, float maxY) {
		int length = this.length;
		if (length == capacity()) this.grow();
		var buffer = this.buffer;
		var addr = buffer.address() + length * LAYOUT.byteSize();
		// using Unsafe to avoid bounds checking
		UnsafeWrapper.putFloat(null, addr + Float.BYTES * 0, x);
		UnsafeWrapper.putFloat(null, addr + Float.BYTES * 1, y);
		UnsafeWrapper.putFloat(null, addr + Float.BYTES * 2, maxX);
		UnsafeWrapper.putFloat(null, addr + Float.BYTES * 3, maxY);
//		VH_x.set(buffer, (long) length, x);
//		VH_y.set(buffer, (long) length, y);
//		VH_width.set(buffer, (long) length, width);
//		VH_height.set(buffer, (long) length, height);
		this.length = length + 1;
	}

	public void pop() {
		this.length--;
	}

	private long topI() {
		assert this.length > 0;
		return this.length - 1;
	}

	private long topAddr() {
		return this.buffer.address() + topI() * LAYOUT.byteSize();
	}

	/**
	 * This is safe to call only when {@link #isEmpty()} returns {@code false}.
	 */
	public float topMinX() {
		return UnsafeWrapper.getFloat(topAddr() + Float.BYTES * 0);
//		return (float) VH_x.get(buffer, this.topI());
	}

	/**
	 * This is safe to call only when {@link #isEmpty()} returns {@code false}.
	 */
	public float topMinY() {
//		return (float) VH_y.get(buffer, this.topI());
		return UnsafeWrapper.getFloat(topAddr() + Float.BYTES * 1);
	}

	/**
	 * This is safe to call only when {@link #isEmpty()} returns {@code false}.
	 */
	public float topMaxX() {
//		return (float) VH_width.get(buffer, this.topI());
		return UnsafeWrapper.getFloat(topAddr() + Float.BYTES * 2);
	}

	/**
	 * This is safe to call only when {@link #isEmpty()} returns {@code false}.
	 */
	public float topMaxY() {
//		return (float) VH_height.get(buffer, this.topI());
		return UnsafeWrapper.getFloat(topAddr() + Float.BYTES * 3);
	}

	public boolean isEmpty() {
		return this.length == 0;
	}

	public int length() {
		return this.length;
	}
}
