package dev.pfaff.altargui.internal.util;

import java.io.FileDescriptor;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.foreign.MemorySegment;
import java.lang.foreign.ValueLayout;
import java.nio.ByteBuffer;

public final class DumpUtil {
	private DumpUtil() {}

	private static final String HEX_CHARS = "0123456789abcdef";

	public static StringBuilder appendHex(StringBuilder sb, byte b) {
		return sb.append(HEX_CHARS.charAt((b >>> 4) & 0xF))
				 .append(HEX_CHARS.charAt(b & 0xF));
	}

	public static void dumpMemory(MemorySegment ms) {
		long n = ms.byteSize();

		var sb = new StringBuilder();
		int iMaxDigits = Long.toString(n).length();
		try {
			// TODO: optimize this (avoid sb.toString()).
			@SuppressWarnings("resource")
			var wtr = new FileWriter(FileDescriptor.out);
			for (int i = 0; i < n; ) {
				int iDigits = i == 0 ? 1 : ((int)Math.floor(Math.log10(i)) + 1);
				sb.repeat(' ', iMaxDigits - iDigits);
				sb.append(i);
				sb.append(" |");
				int j;
				for (j = i, i += 4 * 8; j < i && j < n;) {
					int k;
					for (k = j, j += 8; k < j && k < n; k++) {
						sb.append(' ');
						appendHex(sb, ms.get(ValueLayout.JAVA_BYTE, k));
					}
					sb.append("  ");
				}
				sb.append('\n');
				wtr.write(sb.toString());
				sb.setLength(0);
			}
			wtr.flush();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public static void dumpByteBuffer(ByteBuffer buffer) {
		dumpMemory(MemorySegment.ofBuffer(buffer));
	}
}
