package dev.pfaff.altargui.internal.util;

import java.lang.foreign.Arena;
import java.lang.foreign.MemorySegment;

public final class GrowableMemorySegment {
	private MemorySegment ms;

	public GrowableMemorySegment() {
		this.ms = MemorySegment.NULL;
	}

	public GrowableMemorySegment(long initialCapacity) {
		this();
		if (initialCapacity != 0) {
			this.growAlways(initialCapacity);
		}
	}

	private void growAlways(long minCapacity) {
		this.ms = growAlways(this.ms, minCapacity, Long.BYTES);
	}

	public static MemorySegment growAlways(MemorySegment segment, long minCapacity, long byteAlignment) {
		var capacity = Math.max(segment.byteSize(), 1);
		while (capacity < minCapacity) {
			capacity *= 2;
		}
		return Arena.ofAuto().allocate(capacity, byteAlignment).copyFrom(segment);
	}

	public void ensureTotalCapacity(long minCapacity) {
		if (minCapacity > this.ms.byteSize()) growAlways(minCapacity);
	}

	public MemorySegment ms() {
		return this.ms;
	}
}
