package dev.pfaff.altargui.libimage;

import dev.pfaff.glass.opengl.enums.GLPixelFormat;
import dev.pfaff.glass.opengl.enums.GLPixelType;
import org.jspecify.annotations.Nullable;

import java.util.List;

public enum ColorType {
	// order matters
	L8,
	La8,
	Rgb8,
	Rgba8,
	L16,
	La16,
	Rgb16,
	Rgba16,
	Rgb32F,
	Rgba32F;

	public static final List<ColorType> VALUES = List.of(values());

	public final byte value() {
		return (byte) ordinal();
	}

	public ExtendedColorType extended() {
		return switch (this) {
			case L8 -> ExtendedColorType.L8;
			case La8 -> ExtendedColorType.La8;
			case Rgb8 -> ExtendedColorType.Rgb8;
			case Rgba8 -> ExtendedColorType.Rgba8;
			case L16 -> ExtendedColorType.L16;
			case La16 -> ExtendedColorType.La16;
			case Rgb16 -> ExtendedColorType.Rgb16;
			case Rgba16 -> ExtendedColorType.Rgba16;
			case Rgb32F -> ExtendedColorType.Rgb32F;
			case Rgba32F -> ExtendedColorType.Rgba32F;
		};
	}

	/**
	 * @return the size of each pixel in this color type, in bytes.
	 */
	public int pixelSize() {
		return switch (this) {
			case L8 -> 1;
			case La8 -> 2;
			case Rgb8 -> 3;
			case Rgba8 -> 4;
			case L16 -> 2;
			case La16 -> 4;
			case Rgb16 -> 6;
			case Rgba16 -> 8;
			case Rgb32F -> 12;
			case Rgba32F -> 16;
		};
	}

	public final GLPixelFormat glPixelFormat() {
		return switch (this) {
			case La8, La16 -> GLPixelFormat.LUMINANCE_ALPHA;
			case L8, L16 -> GLPixelFormat.LUMINANCE;
			case Rgb8, Rgb16, Rgb32F -> GLPixelFormat.RGB;
			case Rgba8, Rgba16, Rgba32F -> GLPixelFormat.RGBA;
		};
	}

	public final GLPixelType glChannelType() {
		return switch (this) {
			case Rgba8 -> GLPixelType.UNSIGNED_INT_8_8_8_8_REV;
			case L8, La8, Rgb8 -> GLPixelType.UNSIGNED_BYTE;
			case L16, La16, Rgb16, Rgba16 -> GLPixelType.UNSIGNED_SHORT;
			case Rgb32F, Rgba32F -> GLPixelType.UNSIGNED_INT;
		};
	}
}
