package dev.pfaff.altargui.pathfinder;

import java.lang.foreign.Arena;
import java.lang.foreign.MemorySegment;

import static dev.pfaff.altargui.pathfinder.Pathfinder.*;
import static java.lang.foreign.MemoryLayout.PathElement.groupElement;
import static java.lang.foreign.ValueLayout.*;

public record PFGLRendererRef(MemorySegment ptr) {
	/**
	 * @param device moved into the renderer
	 * @param resourceLoader borrowed by the renderer
	 */
	public static PFGLRendererRef create(PFGLDeviceRef device,
										 PFResourceLoaderRef resourceLoader,
										 PFRendererLevel level,
										 PFGLDestFramebufferInit framebuffer) {
		return new PFGLRendererRef(constructNativeObject(ML_PFGLRenderer,
				ptr -> {
					try (var arena = Arena.ofConfined()) {
						var modePtr = arena.allocateFrom(JAVA_BYTE, level.value).asSlice(0, ML_PFRendererMode);
						var options = arena.allocate(ML_PFRendererOptions);
						framebuffer.init(options.asSlice(ML_PFRendererOptions.byteOffset(groupElement(0)),
								ML_PFDestFramebufferRef));
						options.set(JAVA_FLOAT,
								ML_PFRendererOptions.byteOffset(groupElement(1), groupElement("r")),
								1f);
						options.set(JAVA_FLOAT,
								ML_PFRendererOptions.byteOffset(groupElement(1), groupElement("g")),
								1f);
						options.set(JAVA_FLOAT,
								ML_PFRendererOptions.byteOffset(groupElement(1), groupElement("b")),
								1f);
						options.set(JAVA_FLOAT,
								ML_PFRendererOptions.byteOffset(groupElement(1), groupElement("a")),
								1f);
						// 0b1 : enable background color
						// 0b10 : enable debug ui
						options.set(JAVA_BYTE, ML_PFRendererOptions.byteOffset(groupElement(2)), (byte) 0b00);
						return (int) MH_PFGLRendererInit.invokeExact(ptr,
								device.ptr(),
								resourceLoader.ptr(),
								modePtr,
								options);
					}
				},
				ptr -> MH_PFGLRendererDeinit.invokeExact(ptr)));
	}

	public void destFramebufferSizeChanged() {
		try {
			MH_PFGLRendererDestFramebufferSizeChanged.invokeExact(this.ptr);
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public void setViewport(int x, int y, int width, int height) {
		try (var arena = Arena.ofConfined()) {
			var rect = arena.allocateFrom(JAVA_INT, x, y, width, height).asSlice(0, ML_PFRectI);
			if ((int) MH_PFGLRendererSetViewport.invokeExact(this.ptr, rect) != 0) {
				throw new RuntimeException();
			}
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public void setWindowSize(int width, int height) {
		try (var arena = Arena.ofConfined()) {
			var size = arena.allocateFrom(JAVA_INT, width, height).asSlice(0, ML_PFVector2I);
			if ((int) MH_PFGLRendererSetWindowSize.invokeExact(this.ptr, size) != 0) {
				throw new RuntimeException();
			}
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}
}
