package dev.pfaff.altargui.pathfinder.fontkit;

import org.jspecify.annotations.Nullable;

import java.lang.foreign.Arena;
import java.lang.foreign.MemorySegment;

import static dev.pfaff.altargui.pathfinder.Pathfinder.MH_FKFontGlyphForCodepoint;
import static dev.pfaff.altargui.pathfinder.Pathfinder.MH_FKFontRasterizeGlyph;
import static java.lang.foreign.ValueLayout.JAVA_INT;

public record FKFontRef(MemorySegment ptr) {
	@Nullable
	public Integer glyphForCodepoint(int codepoint) {
		try {
			var result = (long) MH_FKFontGlyphForCodepoint.invokeExact(ptr(), codepoint);
			if ((result >>> 32) != 0) {
				return (int) result;
			} else {
				return null;
			}
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public void rasterizeCodepoint(MemorySegment canvasPixelBuffer, int canvasWidth, int canvasHeight, FKCanvasFormat canvasFormat, int glyphId, float pointSize, FKCanvasRasterizationOptions rasterizationOptions) {
		try (var arena = Arena.ofConfined()) {
			var canvasSize = arena.allocateFrom(JAVA_INT, canvasWidth, canvasHeight);
			var result = (int) MH_FKFontRasterizeGlyph.invokeExact(ptr(),
					canvasPixelBuffer,
					canvasPixelBuffer.byteSize(),
					canvasSize,
					canvasFormat.value(),
					glyphId,
					pointSize,
					rasterizationOptions.value());
			if (result != 0) {
				throw new RuntimeException();
			}
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}
}
