package dev.pfaff.altargui.pathfinder.fontkit;

import dev.pfaff.altargui.libimage.ColorType;
import dev.pfaff.altargui.libimage.ImageData;
import dev.pfaff.altargui.libimage.ImageInfo;
import dev.pfaff.altargui.texture.LoadableTexture;
import dev.pfaff.altargui.util.GLThread;

import java.io.IOException;
import java.lang.foreign.Arena;
import java.lang.foreign.ValueLayout;

import static dev.pfaff.altargui.util.BasicFuture.executeOn;

public record FKGlyphLoadableTexture(FKFontRef font, int codepoint) implements LoadableTexture {
	private static final int SIZE = 32;

	@Override
	public ImageData load() throws IOException {
		// inspection is misguided
		//noinspection DataFlowIssue
		var idOrNull = executeOn(() -> this.font.glyphForCodepoint(this.codepoint), GLThread::submitToGLThread);
		// inspection is wrong
		//noinspection ConstantValue
		if (idOrNull == null) throw new IOException("The font does not contain the specified codepoint");
		int id = idOrNull;
		var buf = Arena.ofAuto().allocate(ValueLayout.JAVA_INT, SIZE * SIZE);
		executeOn(() -> this.font.rasterizeCodepoint(buf,
				SIZE,
				SIZE,
				FKCanvasFormat.Rgba32,
				id,
				24f,
				FKCanvasRasterizationOptions.GrayscaleAa), GLThread::submitToGLThread);
		return new ImageData(new ImageInfo(SIZE, SIZE, ColorType.Rgba8), buf);
	}
}
