package dev.pfaff.altargui.render;

import dev.pfaff.altargui.uploadables.Uploadable;
import dev.pfaff.glass.opengl.enums.GLBufferTarget;

import java.lang.foreign.Arena;
import java.lang.foreign.MemorySegment;

public abstract class AbstractMappedBuffer extends RawBuffer {
	private final boolean isShared;
	protected MemorySegment mappedBuffer;
	protected Arena mapArena;
	protected int capacity;

	public AbstractMappedBuffer(GLBufferTarget bindTarget,
								boolean isShared) {
		super(bindTarget);
		this.isShared = isShared;
	}

	protected final Arena newArena() {
		return isShared ? Arena.ofShared() : Arena.ofConfined();
	}

	@Override
	public final int capacity() {
		return this.capacity;
	}

	@Override
	public boolean isBoundDuringUpload() {
		return true;
	}

	@Override
	public final void uploadAtUnchecked(long offset, Uploadable uploadable) {
		var sink = new MemorySegmentSink(mappedBuffer, offset);
		uploadable.uploadTo(sink);
		checkCompleteUpload(sink.offset() - offset, uploadable);
	}

	@Override
	public void close() {
		if (mapArena != null) mapArena.close();
		super.close();
	}

	@Override
	public String toString() {
		var s = new StringBuilder(super.toString());
		s.setLength(s.length() - 1);
		return s.append(", mapped=").append(mapArena != null).append(']').toString();
	}
}
