package dev.pfaff.altargui.render;

import dev.pfaff.altargui.types.Color;
import dev.pfaff.log4truth.NamedLogger;

import java.nio.ByteOrder;
import java.util.List;

import static dev.pfaff.log4truth.StandardTags.*;
import static java.lang.foreign.ValueLayout.*;
import static java.util.FormatProcessor.FMT;

public final class BufferUtil {
	// TODO: `LOGGER.` should be replaced with `Logger.` once I patch the JDK to allow the use of
	//  `Reflection.getCallerClass` which is (supposedly) very fast.
	private static final NamedLogger LOGGER = NamedLogger.forCaller();

	private static final List<String> TAGS = List.of(DEBUG);

	public static void emitVec2(Sink buffer, float x, float y) {
		LOGGER.log(() -> STR."emitVec2(\{x}, \{y})", TAGS);
//		buffer.accept(packFloats(x, y));
		buffer.checkRemaining(JAVA_FLOAT_UNALIGNED.byteSize() * 2);
		buffer.accept(x);
		buffer.accept(y);
	}

	public static void emitVec2AsNormUShort(Sink buffer, float x, float y) {
		LOGGER.log(() -> STR."emitVec2AsNormUShort(\{x}, \{y})", TAGS);
		emitCVec2(buffer, (char)(x * Character.MAX_VALUE), (char)(y * Character.MAX_VALUE));
	}

	public static void emitVec2AsNormUByte(Sink buffer, float x, float y) {
		LOGGER.log(() -> STR."emitVec2AsNormUByte(\{x}, \{y})", TAGS);
		emitBVec2(buffer, (byte)(int)(x * 256f), (byte)(int)(y * 256f));
	}

	public static void emitCVec2(Sink buffer, char x, char y) {
		LOGGER.log(() -> STR."emitCVec2(\{x}, \{y})", TAGS);
		buffer.checkRemaining(JAVA_CHAR_UNALIGNED.byteSize() * 2);
		buffer.accept(x);
		buffer.accept(y);
//		buffer.accept(packChars(x, y));
	}

	public static void emitBVec2(Sink buffer, byte x, byte y) {
		LOGGER.log(() -> STR."emitBVec2(\{x}, \{y})", TAGS);
		buffer.checkRemaining(JAVA_BYTE.byteSize() * 2);
		buffer.accept(x);
		buffer.accept(y);
//		buffer.accept(packChars(x, y));
	}

	public static void emitUnitQuadVertices(Sink buffer) {
		LOGGER.log(() -> "emitUnitQuadVertices()", TAGS);
		buffer.checkRemaining(JAVA_FLOAT_UNALIGNED.byteSize() * 2 * 6);
		emitVec2(buffer, 0f, 1f); // Top-left
		emitVec2(buffer, 1f, 1f); // Top-right
		emitVec2(buffer, 1f, 0f); // Bottom-right

		emitVec2(buffer, 1f, 0f); // Bottom-right
		emitVec2(buffer, 0f, 0f); // Bottom-left
		emitVec2(buffer, 0f, 1f); // Top-left
	}

	public static void emitShaderColor(Sink buffer, Color color) {
		LOGGER.log(() -> FMT."emitShaderColor(\{color})", TAGS);
		buffer.accept(ByteOrder.BIG_ENDIAN, color.rgba());
	}

	public static long packFloats(float x, float y) {
		long xBits = Float.floatToRawIntBits(x);
		long yBits = Float.floatToRawIntBits(y);
		return ByteOrder.nativeOrder() == ByteOrder.LITTLE_ENDIAN ? xBits | yBits << Float.SIZE
																  : yBits | xBits << Float.SIZE;
	}

	public static float unpackFloatX(long bits) {
		return Float.intBitsToFloat(ByteOrder.nativeOrder() == ByteOrder.LITTLE_ENDIAN
				? (int) bits
				: (int) (bits >>> 32));
	}

	public static float unpackFloatY(long bits) {
		return Float.intBitsToFloat(ByteOrder.nativeOrder() == ByteOrder.LITTLE_ENDIAN
				? (int) (bits >>> 32)
				: (int) bits);
	}

	public static int packChars(char x, char y) {
		int xBits = x;
		int yBits = y;
		return ByteOrder.nativeOrder() == ByteOrder.LITTLE_ENDIAN ? xBits | yBits << Short.SIZE
																  : yBits | xBits << Short.SIZE;
	}
}
