package dev.pfaff.altargui.render;

import java.lang.foreign.MemorySegment;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public final class ByteBufferSink implements Sink {
	private final ByteBuffer buffer;

	public ByteBufferSink(ByteBuffer buffer) {
		this.buffer = buffer;
	}

	@Override
	public void checkRemaining(long length) {
		if (buffer.remaining() < length) {
			throw new BufferOverflowException(STR."Overflowed buffer by \{length - buffer.remaining()} bytes");
		}
	}

	@Override
	public void accept(byte value) {
		checkRemaining(Byte.BYTES);
		buffer.put(value);
	}

	@Override
	public void accept(short value) {
		checkRemaining(Short.BYTES);
		buffer.putShort(value);
	}

	@Override
	public void accept(char value) {
		checkRemaining(Character.BYTES);
		buffer.putChar(value);
	}

	@Override
	public void accept(int value) {
		checkRemaining(Integer.BYTES);
		buffer.putInt(value);
	}

	@Override
	public void accept(ByteOrder order, int value) {
		checkRemaining(Integer.BYTES);
		buffer.order(order).putInt(value).order(ByteOrder.nativeOrder());
	}

	@Override
	public void accept(long value) {
		checkRemaining(Long.BYTES);
		buffer.putLong(value);
	}

	@Override
	public void accept(float value) {
		checkRemaining(Float.BYTES);
		buffer.putFloat(value);
	}

	@Override
	public void accept(double value) {
		checkRemaining(Double.BYTES);
		buffer.putDouble(value);
	}

	@Override
	public void accept(MemorySegment ms) {
		checkRemaining(ms.byteSize());
		MemorySegment.ofBuffer(this.buffer).copyFrom(ms);
	}

	@Override
	public void accept(ByteBuffer buffer, int offset, int length) {
		checkRemaining(length);
		var pos = this.buffer.position();
		this.buffer.put(pos, buffer, offset, length).position(pos + length);
	}

	@Override
	public String toString() {
		return STR."ByteBufferSink[buffer=\{buffer}]";
	}
}
