package dev.pfaff.altargui.render;

import dev.pfaff.altargui.shader.BuiltInstanceLayout;
import dev.pfaff.glass.opengl.enums.GLBufferTarget;

import java.io.Closeable;
import java.util.List;
import java.util.stream.Stream;

public final class FlipBuffer implements Closeable {
	private final List<Buffer> buffers;
	private int current = 0;

	public FlipBuffer(int count, BuiltInstanceLayout instanceDef, GLBufferTarget bindTarget, boolean isShared) {
		this.buffers = Stream.generate(() -> new Buffer(instanceDef, RawBuffer.create(bindTarget, isShared))).limit(count).toList();
	}

	public Buffer get() {
		return buffers.get(current);
	}

	public void flip() {
		current = (current + 1) % buffers.size();
	}

	@Override
	public void close() {
		buffers.forEach(Buffer::close);
	}

	@Override
	public String toString() {
		return STR."FlipBuffer[current=\{current}, buffers=\{buffers}]";
	}
}
