package dev.pfaff.altargui.render;

import dev.pfaff.altargui.shader.Program;
import dev.pfaff.glass.opengl.GLException;
import dev.pfaff.glass.opengl.enums.GLErrorCode;
import it.unimi.dsi.fastutil.objects.ObjectArrayList;

import java.lang.foreign.MemorySegment;

import static java.lang.foreign.ValueLayout.JAVA_INT;

/**
 * Uses {@link GLUtil#glDrawArraysInstanced}.
 */
public final class GL31LayeredRenderBuffer extends LayeredRenderBuffer {
	private final ObjectArrayList<VAO> vaos = ObjectArrayList.wrap(VAO.EMPTY_ARRAY);
	private int nextVAO = 0;

	public GL31LayeredRenderBuffer() {
	}

	@Override
	public void beginUploading(int capacity) {
		super.beginUploading(capacity);
		this.nextVAO = 0;
	}

	@Override
	protected void onRecreate() {
	}

	@Override
	public void close() {
		this.vaos.forEach(AbstractVertexArrayObject::close);
		super.close();
	}

	@Override
	protected void putDataForEntry(MemorySegment ms, int programId, int start, int end) {
		var layout = this.instanceLayouts.get(programId);
		int vaoI = this.nextVAO++;
		if (vaoI == this.vaos.size()) {
			this.vaos.push(new VAO());
		}
		var vao = this.vaos.get(vaoI);
		try (var _ = vao.bound()) {
			try (var _ = this.instancesBuffer.boundIfNotAlreadyDuringUpload()) {
				layout.bindAttribsWithOffset(start * layout.stride());
			}
		}
		ms.setAtIndex(JAVA_INT, 0, vaoI);
		ms.setAtIndex(JAVA_INT, 1, end - start);
	}

	@Override
	protected VAO getVAOForEntry(MemorySegment ms, int programId) {
		return this.vaos.get(ms.getAtIndex(JAVA_INT, 0));
	}

	@Override
	protected void drawEntry(MemorySegment ms, int programId, Program<?> program) {
		var drawMode = program.drawMode();
		var templateVertexCount = program.templateVertexCount();
		try {
			int instanceCount = ms.getAtIndex(JAVA_INT, 1);
			GLUtil.glDrawArraysInstanced(drawMode.glPrimitiveType(),
					0,
					templateVertexCount,
					instanceCount);
		} catch (GLException e) {
			if (e.code == GLErrorCode.INVALID_OPERATION) {
				throw new RuntimeException(STR."Either a geometry shader is active or one of the buffers (\{this}) is mapped and not persistent", e);
			} else {
				throw e;
			}
		}
	}
}
