package dev.pfaff.altargui.render;

import dev.pfaff.altargui.shader.Program;
import dev.pfaff.glass.opengl.GLException;
import dev.pfaff.glass.opengl.enums.GLErrorCode;
import dev.pfaff.log4truth.Logger;

import java.lang.foreign.MemorySegment;
import java.util.List;
import java.util.stream.IntStream;

import static java.lang.foreign.ValueLayout.JAVA_INT;

/**
 * Uses {@link GLUtil#glDrawArraysInstanced}.
 */
public final class GL31LayeredRenderBufferNew extends LayeredRenderBuffer {
	private final List<VAO> vaos = IntStream.range(0, this.instanceLayouts.size())
			.mapToObj(_ -> new VAO())
			.toList();

	public GL31LayeredRenderBufferNew() {
	}

	@Override
	public void beginUploading(int capacity) {
		super.beginUploading(capacity);
	}

	@Override
	protected void onRecreate() {
	}

	@Override
	public void close() {
		this.vaos.forEach(AbstractVertexArrayObject::close);
		super.close();
	}

	@Override
	protected void putDataForEntry(MemorySegment ms, int programId, int start, int end) {
		int instanceCount = end - start;
		var layout = this.instanceLayouts.get(programId);
//		Logger.log(() -> STR."put entry program=\{programId} start=\{start * layout.stride()} length=\{instanceCount * layout.stride()}");
		ms.setAtIndex(JAVA_INT, 0, start * layout.stride());
		ms.setAtIndex(JAVA_INT, 1, instanceCount);
	}

	@Override
	protected VAO getVAOForEntry(MemorySegment ms, int programId) {
		return this.vaos.get(programId);
	}

	@Override
	protected void drawEntry(MemorySegment ms, int programId, Program<?> program) {
		var drawMode = program.drawMode();
		var templateVertexCount = program.templateVertexCount();
		try {
			int first = ms.getAtIndex(JAVA_INT, 0);

			try (var _ = this.instancesBuffer.boundIfNotAlreadyDuringUpload()) {
				this.instanceLayouts.get(programId).bindAttribsWithOffset(first);
			}

			int instanceCount = ms.getAtIndex(JAVA_INT, 1);
//			Logger.log(() -> STR."draw batch drawMode=\{drawMode} first=\{first} templateVertexCount=\{templateVertexCount} instanceCount=\{instanceCount} length=\{instanceCount * program.instanceLayout().stride()}");
			GLUtil.glDrawArraysInstanced(drawMode.glPrimitiveType(),
					0,
					templateVertexCount,
					instanceCount);
		} catch (GLException e) {
			if (e.code == GLErrorCode.INVALID_OPERATION) {
				throw new RuntimeException(STR."Either a geometry shader is active or one of the buffers (\{this}) is mapped and not persistent", e);
			} else {
				throw e;
			}
		}
	}
}
