package dev.pfaff.altargui.render;

import dev.pfaff.altargui.GuiGlobals;
import dev.pfaff.altargui.shader.Program;
import dev.pfaff.glass.opengl.GLException;
import dev.pfaff.glass.opengl.enums.GLErrorCode;
import dev.pfaff.glass.opengl.enums.GLPrimitiveType;

import java.lang.foreign.MemorySegment;
import java.util.List;
import java.util.stream.Stream;

import static java.lang.foreign.ValueLayout.JAVA_INT;

/**
 * Uses {@link GLUtil#glDrawArraysInstancedBaseInstance(GLPrimitiveType, int, int, int, int)}.
 */
public final class GL42DrawArraysInstancedBaseInstanceLayeredRenderBuffer extends LayeredRenderBuffer {
	// TODO: use explicit attribute locations so that we can use a single VAO for everything!
	private final List<VAO> vaos = Stream.generate(VAO::new).limit(this.programs.size()).toList();

	public GL42DrawArraysInstancedBaseInstanceLayeredRenderBuffer() {
	}

	@Override
	protected void onRecreate() {
		var vaosIter = this.vaos.iterator();
		var layoutsIter = this.instanceLayouts.iterator();
		while (vaosIter.hasNext()) {
			@SuppressWarnings("resource") var vao = vaosIter.next();
			var layout = layoutsIter.next();
			vao.bind();
			try {
				layout.bindAttribs();
			} finally {
				vao.unbind();
			}
		}
	}

	@Override
	public void close() {
		this.vaos.forEach(AbstractVertexArrayObject::close);
		super.close();
	}

	@Override
	protected void putDataForEntry(MemorySegment ms, int programId, int start, int end) {
		ms.setAtIndex(JAVA_INT, 0, start);
		ms.setAtIndex(JAVA_INT, 1, end - start);
	}

	@Override
	protected VAO getVAOForEntry(MemorySegment ms, int programId) {
		return this.vaos.get(programId);
	}

	@Override
	protected void drawEntry(MemorySegment ms, int programId, Program program) {
		var drawMode = program.drawMode();
		var templateVertexCount = program.templateVertexCount();
		try {
			int start = ms.getAtIndex(JAVA_INT, 0);
			int instanceCount = ms.getAtIndex(JAVA_INT, 1);
			GuiGlobals.GL.glDrawArraysInstancedBaseInstance(drawMode.glPrimitiveType(),
					0,
					templateVertexCount,
					instanceCount,
					start);
		} catch (GLException e) {
			if (e.code == GLErrorCode.INVALID_OPERATION) {
				throw new RuntimeException(STR."Either a geometry shader is active or one of the buffers (\{this}) is mapped and not persistent", e);
			} else {
				throw e;
			}
		}
	}
}
