package dev.pfaff.altargui.render;

import dev.pfaff.altargui.GuiGlobals;
import dev.pfaff.altargui.internal.Config;
import dev.pfaff.altargui.internal.util.DumpUtil;
import dev.pfaff.glass.opengl.enums.GLBufferAccess;
import dev.pfaff.glass.opengl.enums.GLBufferStorageTarget;
import dev.pfaff.glass.opengl.enums.GLBufferTarget;

import java.lang.foreign.Arena;
import java.lang.foreign.MemorySegment;

import static dev.pfaff.altargui.GuiGlobals.LOGGER;
import static dev.pfaff.glass.opengl.enums.GLBufferStorageMask.MAP_PERSISTENT_BIT;
import static dev.pfaff.glass.opengl.enums.GLBufferStorageMask.MAP_WRITE_BIT;
import static dev.pfaff.glass.opengl.enums.GLMapBufferAccessMask.MAP_INVALIDATE_BUFFER_BIT;

public final class PersistentlyMappedBuffer extends AbstractMappedBuffer {
	private int id = -1;

	public PersistentlyMappedBuffer(GLBufferTarget bindTarget, boolean isShared) {
		super(bindTarget, isShared);
	}

	@Override
	public void growBuffer(int capacity, Runnable onRecreate) {
		var newCapacity = this.capacity == 0 ? 2 : this.capacity;
		while (newCapacity < capacity) {
			newCapacity *= 2;
		}

		int newCapacity_ = newCapacity;
		if (mapArena != null) mapArena.close();
		// new arena, invalidate the old
		mapArena = newArena();
		if (id != -1) GLUtil.glDeleteBuffers(id);
		id = GLUtil.glGenBuffers();
		GuiGlobals.GL.glBindBuffer(bindTarget, id);
		try {
			GLUtil.glBufferStorage(GLBufferStorageTarget.fromGLEnum(bindTarget.glEnum()),
					newCapacity_,
					MAP_WRITE_BIT.glEnum() | MAP_PERSISTENT_BIT.glEnum());
			onRecreate.run();
			// unmapping is automatic.
			mappedBuffer = GLUtil.glMapBufferRange(bindTarget,
							0,
							newCapacity_,
							MAP_WRITE_BIT.glEnum() | MAP_PERSISTENT_BIT.glEnum() | MAP_INVALIDATE_BUFFER_BIT.glEnum())
					.reinterpret(mapArena, ignored -> {});
		} finally {
			if (Config.GL_ALWAYS_UNBIND) GuiGlobals.GL.glBindBuffer(bindTarget, 0);
		}

		this.capacity = newCapacity_;
	}

	@Override
	protected void resetNoGrow() {
	}

	public int id() {
		return id;
	}

	public void dump(String label, boolean capacity) {
		try (var arena = Arena.ofConfined()) {
			GuiGlobals.GL.glBindBuffer(GLBufferTarget.COPY_READ_BUFFER, id);
			MemorySegment buffer = GLUtil.glMapBuffer(GLBufferTarget.COPY_READ_BUFFER,
							GLBufferAccess.READ_ONLY,
							capacity ? this.capacity : length())
					.reinterpret(arena, ignored -> {
						GLUtil.glUnmapBuffer(GLBufferTarget.COPY_READ_BUFFER);
						if (Config.GL_ALWAYS_UNBIND)
							GuiGlobals.GL.glBindBuffer(GLBufferTarget.COPY_READ_BUFFER, 0);
					});
			LOGGER.log(() -> STR."\{label}:  size: \{length()} bytes, capacity: \{capacity} bytes, buffer size: \{buffer.byteSize()} bytes");
			DumpUtil.dumpMemory(buffer);
		}
	}
}
