package dev.pfaff.altargui.render;

import dev.pfaff.altargui.CanvasImpl;
import dev.pfaff.altargui.internal.util.Specialize;
import dev.pfaff.altargui.render.objects.StaticCPUBuffer;
import dev.pfaff.altargui.shader.InstanceLayout;
import dev.pfaff.altargui.uploadables.Uploadable;

import java.lang.foreign.MemoryLayout;
import java.util.function.Consumer;

/**
 * Most use cases should be satisfied by {@link #buildSingleStatic}. However, if you require rapidly updating data that is
 * specific to each instance, rather than each program (in which case you should use a custom shader and uniforms
 * instead), then a custom implementation may be warranted.
 */
public abstract class RenderObject implements Uploadable, AutoCloseable {
	public static final RenderObject[] EMPTY_ARRAY = new RenderObject[0];

	public abstract int instanceCount();

	public abstract MemoryLayout instanceLayout();

	@Specialize
	@Override
	public int size() {
		return (int) instanceLayout().byteSize() * instanceCount();
	}

	public abstract int program();

	@Override
	public abstract void close();

	public static RenderObject buildStatic(int layer, MemoryLayout instanceStride, int instanceCount, Consumer<? super ByteBufferSink> builder) {
		// testing currently bizarrely shows CPU buffers to be faster than GPU buffers.
		return StaticCPUBuffer.build(layer, instanceStride, instanceCount, builder);
	}

	public static RenderObject buildSingleStatic(int layer, InstanceLayout spec, Consumer<Sink> builder) {
		return buildStatic(layer, spec.layout(), 1, builder);
	}

	@Specialize
	public void uSpecUploadCanvas(CanvasImpl canvas) {
		canvas.iSpecUpload(this, this.size());
	}

	@Specialize
	public void uSpecUploadLayered(LayeredRenderBuffer buffer) {
		buffer.iSpecUpload(this, this.program());
	}
}
