package dev.pfaff.altargui.render;

import dev.pfaff.altargui.uploadables.Uploadable;
import dev.pfaff.glass.opengl.enums.GLBufferTarget;
import dev.pfaff.glass.opengl.enums.GLBufferUsage;

import java.lang.foreign.MemorySegment;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public final class ViaBufferSubDataBuffer extends RawBuffer {
	private final int id;
	// @Stable
	private boolean initialized;
	private int capacity;

	public ViaBufferSubDataBuffer(GLBufferTarget bindTarget) {
		super(bindTarget);
		id = GLUtil.glGenBuffers();
	}

	@Override
	public int capacity() {
		return this.capacity;
	}

	@Override
	public boolean isBoundDuringUpload() {
		return true;
	}

	@Override
	public void growBuffer(int capacity, Runnable onRecreate) {
		var newCapacity = this.capacity == 0 ? 2 : this.capacity;
		while (newCapacity < capacity) {
			newCapacity *= 2;
		}

		this.capacity = newCapacity;

		if (!this.initialized) {
			this.initialized = true;

			bind();
			try {
				onRecreate.run();
			} finally {
				unbind();
			}
		}
	}

	@Override
	protected void resetNoGrow() {
	}

	@Override
	public void beginUploading() {
		super.beginUploading();
		int capacity = this.capacity;
		bind();
		GLUtil.glBufferData(bindTarget, capacity, GLBufferUsage.STREAM_DRAW);
	}

	@Override
	public void finishUploading() {
		unbind();
		super.finishUploading();
	}

	private static final class OurSink implements Sink {
		private final GLBufferTarget target;
		private long offset;

		private OurSink(GLBufferTarget target, long offset) {
			this.target = target;
			this.offset = offset;
		}

		@Override
		public void checkRemaining(long length) {
		}

		@Override
		public void accept(byte value) {
			throw new UnsupportedOperationException();
		}

		@Override
		public void accept(short value) {
			throw new UnsupportedOperationException();
		}

		@Override
		public void accept(char value) {
			throw new UnsupportedOperationException();
		}

		@Override
		public void accept(int value) {
			throw new UnsupportedOperationException();
		}

		@Override
		public void accept(ByteOrder order, int value) {
			throw new UnsupportedOperationException();
		}

		@Override
		public void accept(long value) {
			throw new UnsupportedOperationException();
		}

		@Override
		public void accept(float value) {
			throw new UnsupportedOperationException();
		}

		@Override
		public void accept(double value) {
			throw new UnsupportedOperationException();
		}

		@Override
		public void accept(MemorySegment ms) {
			GLUtil.glBufferSubData(target, offset, ms);
			offset += ms.byteSize();
		}

		@Override
		public void accept(ByteBuffer buffer, int offset, int length) {
			throw new UnsupportedOperationException();
		}
	}

	@Override
	public final void uploadAtUnchecked(long offset, Uploadable uploadable) {
		var sink = new OurSink(bindTarget, offset);
		uploadable.uploadTo(sink);
		checkCompleteUpload(sink.offset - offset, uploadable);
	}

	public int id() {
		return id;
	}
}
