package dev.pfaff.altargui.render;

import dev.pfaff.altargui.uploadables.Uploadable;
import dev.pfaff.glass.opengl.enums.GLBufferTarget;
import dev.pfaff.glass.opengl.enums.GLBufferUsage;

import java.lang.foreign.Arena;
import java.lang.foreign.MemorySegment;

public final class ViaIntermediateBuffer extends RawBuffer {
	private final int id;
	private Arena dataArena;
	private MemorySegment data = MemorySegment.NULL;
	private final boolean isShared;
	// @Stable
	private boolean initialized;

	public ViaIntermediateBuffer(GLBufferTarget bindTarget, boolean isShared) {
		super(bindTarget);
		this.id = GLUtil.glGenBuffers();
		this.isShared = isShared;
	}

	@Override
	public int capacity() {
		return (int)data.byteSize();
	}

	@Override
	public boolean isBoundDuringUpload() {
		return false;
	}

	@Override
	public void growBuffer(int capacity, Runnable onRecreate) {
		var newCapacity = this.capacity() == 0 ? 2 : this.capacity();
		while (newCapacity < capacity) {
			newCapacity *= 2;
		}

		if (this.dataArena != null) this.dataArena.close();
		this.dataArena = isShared ? Arena.ofShared() : Arena.ofConfined();
		this.data = dataArena.allocate(newCapacity, Long.BYTES);

		if (!this.initialized) {
			this.initialized = true;

			bind();
			try {
				onRecreate.run();
			} finally {
				unbind();
			}
		}
	}

	@Override
	protected void resetNoGrow() {
	}

	@Override
	public void finishUploading() {
		bind();
		try {
			GLUtil.glBufferData(bindTarget, data.asSlice(0, length()), GLBufferUsage.STREAM_DRAW);
		} finally {
			unbind();
		}
		super.finishUploading();
	}

	public void iSpecUploadAtUnchecked(long offset, Uploadable uploadable) {
		var sink = new MemorySegmentSink(data, offset);
		uploadable.uploadTo(sink);
		checkCompleteUpload(sink.offset() - offset, uploadable);
	}

	@Override
	public void uploadAtUnchecked(long offset, Uploadable uploadable) {
		uploadable.uSpecUploadAtUnchecked(this, offset);
	}

	public int id() {
		return id;
	}

	@Override
	public void close() {
		this.dataArena.close();
		super.close();
	}
}
