package dev.pfaff.altargui.render.objects;

import dev.pfaff.altargui.CanvasImpl;
import dev.pfaff.altargui.render.LayeredRenderBuffer;
import dev.pfaff.altargui.render.RenderObject;
import dev.pfaff.altargui.render.Sink;
import dev.pfaff.altargui.render.ViaIntermediateBuffer;
import dev.pfaff.altargui.types.Color;

import java.lang.foreign.MemoryLayout;

/**
 * The outlined area, as measured from the inner edges of the outline, will be the specified dimensions.
 */
public final class OutlineRenderObject extends RenderObject {
	// TODO: store these inline
	private final RectRenderObject left;
	private final RectRenderObject right;
	private final RectRenderObject top;
	private final RectRenderObject bottom;

	public OutlineRenderObject() {
		this.left = new RectRenderObject();
		this.right = new RectRenderObject();
		this.top = new RectRenderObject();
		this.bottom = new RectRenderObject();
	}

	public void updatePositionSizeThickness(float x, float y, float width, float height, float thickness) {
		this.top.updatePositionSize(x - thickness, y - thickness, width + thickness * 2, thickness);
		this.bottom.updatePositionSize(x - thickness, y + height, width + thickness * 2, thickness);
		this.left.updatePositionSize(x - thickness, y, thickness, height);
		this.right.updatePositionSize(x + width, y, thickness, height);
	}

	/**
	 * @param color RGBA
	 */
	public void updateColor(Color color) {
		this.left.updateColor(color);
		this.right.updateColor(color);
		this.top.updateColor(color);
		this.bottom.updateColor(color);
	}

	@Override
	public int instanceCount() {
		return this.left.instanceCount() +
			   this.right.instanceCount() +
			   this.top.instanceCount() +
			   this.bottom.instanceCount();
	}

	@Override
	public MemoryLayout instanceLayout() {
		return RectRenderObject.LAYOUT;
	}

	@Override
	public int size() {
		return super.size();
	}

	@Override
	public int program() {
		return this.left.program();
	}

	@Override
	public void close() {
	}

	@Override
	public void uploadTo(Sink sink) {
		this.left.uploadTo(sink);
		this.right.uploadTo(sink);
		this.top.uploadTo(sink);
		this.bottom.uploadTo(sink);
	}

	@Override
	public void uSpecUploadAtUnchecked(ViaIntermediateBuffer buffer, long offset) {
		super.uSpecUploadAtUnchecked(buffer, offset);
	}

	@Override
	public void uSpecUploadCanvas(CanvasImpl canvas) {
		super.uSpecUploadCanvas(canvas);
	}

	@Override
	public void uSpecUploadLayered(LayeredRenderBuffer buffer) {
		super.uSpecUploadLayered(buffer);
	}
}
