package dev.pfaff.altargui.render.objects;

import dev.pfaff.altargui.CanvasImpl;
import dev.pfaff.altargui.render.LayeredRenderBuffer;
import dev.pfaff.altargui.render.Sink;
import dev.pfaff.altargui.render.ViaIntermediateBuffer;

import java.lang.foreign.MemoryLayout;
import java.lang.foreign.StructLayout;

import static dev.pfaff.altargui.render.BufferUtil.emitShaderColor;
import static dev.pfaff.altargui.render.BufferUtil.emitVec2;
import static dev.pfaff.altargui.shader.Programs.RECT_INSTANCE_SPEC;
import static dev.pfaff.altargui.shader.Programs.rectRenderLayer;
import static java.util.FormatProcessor.FMT;

public final class RectRenderObject extends IRectRenderObject {
	public static final StructLayout LAYOUT = RECT_INSTANCE_SPEC.layout();

	@Override
	public MemoryLayout instanceLayout() {
		return LAYOUT;
	}

	@Override
	public int size() {
		return super.size();
	}

	@Override
	public void uploadTo(Sink sink) {
		sink.checkRemaining(LAYOUT.byteSize());
		emitVec2(sink, getX(), getY());
		emitVec2(sink, getWidth(), getHeight());
		emitShaderColor(sink, getColor());
	}

	@Override
	public void uSpecUploadAtUnchecked(ViaIntermediateBuffer buffer, long offset) {
		super.uSpecUploadAtUnchecked(buffer, offset);
	}

	@Override
	public void uSpecUploadCanvas(CanvasImpl canvas) {
		super.uSpecUploadCanvas(canvas);
	}

	@Override
	public void uSpecUploadLayered(LayeredRenderBuffer buffer) {
		super.uSpecUploadLayered(buffer);
	}

	@Override
	public int program() {
		return rectRenderLayer();
	}

	@Override
	public void close() {
	}

	@Override
	public String toString() {
		return FMT."RectRenderObject[x=\{getX()}, y=\{getY()}, width=\{getWidth()}, height=\{getHeight()}, color=%08x\{getColor().rgba()}]";
	}
}
