package dev.pfaff.altargui.render.objects;

import dev.pfaff.altargui.CanvasImpl;
import dev.pfaff.altargui.render.*;

import java.lang.foreign.MemoryLayout;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;
import java.util.Objects;
import java.util.function.Consumer;

public final class StaticCPUBuffer extends RenderObject {
	private final int program;
	private final byte[] buffer;
	private final MemoryLayout instanceLayout;
	private final int instanceCount;

	public StaticCPUBuffer(int program, byte[] buffer, MemoryLayout instanceLayout, int instanceCount) {
		this.program = program;
		this.buffer = buffer;
		this.instanceLayout = instanceLayout;
		this.instanceCount = instanceCount;
	}

	public static StaticCPUBuffer build(int layer, MemoryLayout instanceLayout, int instanceCount, Consumer<? super ByteBufferSink> builder) {
		var buffer = new byte[(int) instanceLayout.byteSize() * instanceCount];

		var byteBuffer = ByteBuffer.wrap(buffer).order(ByteOrder.nativeOrder());
		builder.accept(new ByteBufferSink(byteBuffer));

		if (byteBuffer.position() != buffer.length) {
			throw new RuntimeException(STR."Buffer size not equal to allocated size: expected \{buffer.length}, found \{byteBuffer.position()}");
		}

		return new StaticCPUBuffer(layer, buffer, instanceLayout, instanceCount);
	}

	@Override
	public void uploadTo(Sink sink) {
		sink.accept(buffer);
	}

	@Override
	public void uSpecUploadAtUnchecked(ViaIntermediateBuffer buffer, long offset) {
		super.uSpecUploadAtUnchecked(buffer, offset);
	}

	@Override
	public void uSpecUploadCanvas(CanvasImpl canvas) {
		super.uSpecUploadCanvas(canvas);
	}

	@Override
	public void uSpecUploadLayered(LayeredRenderBuffer buffer) {
		super.uSpecUploadLayered(buffer);
	}

	@Override
	public void close() {
	}

	@Override
	public int program() {
		return program;
	}

	public byte[] buffer() {
		return buffer;
	}

	@Override
	public MemoryLayout instanceLayout() {
		return instanceLayout;
	}

	@Override
	public int instanceCount() {
		return instanceCount;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == this) return true;
		if (obj == null || obj.getClass() != this.getClass()) return false;
		var that = (StaticCPUBuffer) obj;
		return this.program == that.program &&
			   Objects.equals(this.buffer, that.buffer) &&
			   Objects.equals(this.instanceLayout, that.instanceLayout) &&
			   this.instanceCount == that.instanceCount;
	}

	@Override
	public int hashCode() {
		int result = 1;
		result = 31 * result + Integer.hashCode(program);
		result = 31 * result + Arrays.hashCode(buffer);
		result = 31 * result + instanceLayout.hashCode();
		result = 31 * result + Integer.hashCode(instanceCount);
		return result;
	}

	@Override
	public String toString() {
		return STR."StaticCPUBuffer[program=\{program}, buffer=\{buffer}, instanceLayout=\{instanceLayout}, instanceCount=\{instanceCount}]";
	}

}
