package dev.pfaff.altargui.selector;

import dev.pfaff.altargui.WidgetInstance;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.function.Consumer;

/**
 * Selects the single matching element. Fails if more than one element matches.
 * <p>
 * An example usage might be {@code Selector.single(Selector.child())}
 */
public record Single(@Nullable Selector next) implements Selector {
	public Single {
		if (next == Self.INSTANCE) next = null;
	}

	@Override
	public void select(@Nullable WidgetInstance element, Consumer<@NotNull WidgetInstance> consumer) {
		if (next != null) {
			var selected = next.selectSingle(element);
			if (selected != null) consumer.accept(selected);
		} else if (element != null) {
			consumer.accept(element);
		}
	}
}
