package dev.pfaff.altargui.shader;

import dev.pfaff.altargui.render.GLUtil;
import dev.pfaff.glass.opengl.enums.GLVertexAttribIType;
import dev.pfaff.glass.opengl.enums.GLVertexAttribLType;
import dev.pfaff.glass.opengl.enums.GLVertexAttribPointerType;

import java.util.List;

public record BuiltInstanceLayout(int stride, int divisor, List<VertexAttrib> attribs) {
	public BuiltInstanceLayout {
		attribs = List.copyOf(attribs);
	}

	/**
	 * Binds the attributes to whatever buffer is currently active.
	 */
	public void bindAttribs() {
		this.bindAttribsWithOffset(0);
	}

	/**
	 * @param offset the offset of the first element within the buffer.
	 */
	public void bindAttribsWithOffset(int offset) {
		this.attribs.forEach(attrib -> {
			if (attrib == VertexAttrib.PADDING_BYTE) return;
			GLUtil.glEnableVertexAttribArray(attrib.location());
			var attribOffset = offset + attrib.offset();
			switch (attrib.type()) {
				case GLVertexAttribPointerType t -> GLUtil.glVertexAttribPointer(attrib.location(),
						attrib.width(),
						t,
						attrib.normalized(),
						this.stride,
						attribOffset);
				case GLVertexAttribIType t -> GLUtil.glVertexAttribIPointer(attrib.location(),
						attrib.width(),
						t,
						this.stride,
						attribOffset);
				case GLVertexAttribLType t -> GLUtil.glVertexAttribLPointer(attrib.location(),
						attrib.width(),
						t,
						this.stride,
						attribOffset);
			}
			// TODO: we might be able to do this in the constructor instead
			GLUtil.glVertexAttribDivisor(attrib.location(), this.divisor);
		});
	}
}
