package dev.pfaff.altargui.shader;

import dev.pfaff.altargui.util.GLThread;
import dev.pfaff.log4truth.Logger;
import dev.pfaff.glass.opengl.MutPointer;
import dev.pfaff.glass.opengl.enums.GLProgramPropertyARB;
import dev.pfaff.glass.opengl.enums.GLShaderType;

import java.io.IOException;
import java.lang.foreign.Arena;
import java.nio.charset.StandardCharsets;
import java.util.List;

import static dev.pfaff.altargui.GuiGlobals.GL;
import static dev.pfaff.altargui.util.BasicFuture.executeOn;
import static dev.pfaff.log4truth.StandardTags.ERROR;
import static java.lang.foreign.ValueLayout.JAVA_BYTE;
import static java.lang.foreign.ValueLayout.JAVA_INT;

public final class ProgramLoader {
	public static <T extends ShaderProg> T loadProgram(ShaderLoader loader,
													   String vertexShaderName,
													   String fragmentShaderName,
													   ShaderProgConstructor<T> constructor) throws IOException {
		var vertexShader = loader.loadSync(STR."shaders/core/\{vertexShaderName}.vsh", GLShaderType.VERTEX_SHADER);
		var fragmentShader = loader.loadSync(STR."shaders/core/\{fragmentShaderName}.fsh", GLShaderType.FRAGMENT_SHADER);
		return executeOn(() -> {
			var handle = GL.glCreateProgram();
			GL.glAttachShader(handle, vertexShader.handle());
			GL.glAttachShader(handle, fragmentShader.handle());
			GL.glLinkProgram(handle);
			try (var arena = Arena.ofConfined()) {
				var i = arena.allocate(JAVA_INT);
				GL.glGetProgramiv(handle, GLProgramPropertyARB.LINK_STATUS, new MutPointer<>(i));
				if (i.get(JAVA_INT, 0) == 0) {
					var buf = arena.allocate(JAVA_BYTE, 32768);
					var len = arena.allocate(JAVA_INT);
					GL.glGetProgramInfoLog(handle,
							(int) buf.byteSize(),
							new MutPointer<>(len),
							new MutPointer<>(buf));
					var infoLog = buf.getString(0, StandardCharsets.US_ASCII);
					Logger.log(() -> STR."""
Error encountered when linking program containing VS '\{vertexShaderName}' and FS '\{fragmentShaderName}'. Log output:
\{infoLog}""", List.of(ERROR));
				}
			}
			return constructor.construct(vertexShaderName + ":" + fragmentShaderName,
					handle,
					vertexShader,
					fragmentShader);
		}, GLThread::submitToGLThread);
	}
}
