package dev.pfaff.altargui.shader;

import dev.pfaff.altargui.Shaders;

import java.util.List;

import static dev.pfaff.altargui.shader.Attributes.*;
import static dev.pfaff.altargui.shader.VertexAttrib.DIVISOR_PER_INSTANCE;

public final class Programs {
	private static final int rect = 0;
	private static final int texturedRect = 1;
	private static final int text = 2;

	public static final InstanceLayout RECT_INSTANCE_SPEC = new InstanceLayout(List.of(
			ATTRIB_POSITION,
			ATTRIB_SIZE_2,
			ATTRIB_COLOR
	), DIVISOR_PER_INSTANCE);

	public static final InstanceLayout TEXTURED_RECT_INSTANCE_SPEC = new InstanceLayout(List.of(
			ATTRIB_POSITION,
			ATTRIB_SIZE_2,
			ATTRIB_UV,
			ATTRIB_UV_EXTENT,
			ATTRIB_COLOR
	), DIVISOR_PER_INSTANCE);

	public static final InstanceLayout TEXT_INSTANCE_SPEC = new InstanceLayout(List.of(
			ATTRIB_POSITION,
			ATTRIB_SIZE_1,
			ATTRIB_COLOR,
			ATTRIB_CHAR,
			ATTRIB_FLAGS,
			ATTRIB_UV_BASE
	), DIVISOR_PER_INSTANCE);

	public static final Program<ShaderProg2D> RECT_RENDER_LAYER = new Program<>(DrawMode.TRIANGLES,
																		Shaders.RECT_SOLID_COLOR_INSTANCED_SHADER,
																		shader -> {},
																		6,
																		RECT_INSTANCE_SPEC);

	public static final Program<ShaderProg2D> TEXT_RENDER_LAYER = new Program<>(DrawMode.TRIANGLES,
																		Shaders.TEXT_INSTANCED_SHADER,
																		shader -> {},
																		6,
																		TEXT_INSTANCE_SPEC);

	public static final Program<ShaderProg2D> TEXTURED_RECT_RENDER_LAYER = new Program<>(DrawMode.TRIANGLES,
			Shaders.RECT_SOLID_TEX_COLOR_INSTANCED_SHADER,
			shader -> {},
			6,
			TEXTURED_RECT_INSTANCE_SPEC);

	public static int rectRenderLayer() {
		return rect;
	}

	public static int textRenderLayer() {
		return text;
	}

	public static int texturedRectRenderLayer() {
		return texturedRect;
	}
}
