package dev.pfaff.altargui.shader;

import dev.pfaff.altargui.util.GLThread;
import dev.pfaff.log4truth.Logger;

import java.lang.foreign.Arena;
import java.lang.foreign.MemorySegment;
import java.util.List;

import static dev.pfaff.altargui.GuiGlobals.GL;
import static dev.pfaff.log4truth.StandardTags.DEBUG;

/**
 * Garbage-collected shader handle.
 */
public final class Shader {
	private final int handle;
	@SuppressWarnings({"FieldCanBeLocal", "unused"})
	private final MemorySegment ms;

	public Shader(int handle) {
		Logger.log(() -> STR."Created shader \{handle}", List.of(DEBUG));
		this.handle = handle;
		this.ms = MemorySegment.NULL.reinterpret(Arena.ofAuto(), _ -> {
			Logger.log(() -> STR."Freeing shader \{handle}", List.of(DEBUG));
			GLThread.submitToGLThread(() -> GL.glDeleteShader(handle));
		});
	}

	public int handle() {
		return this.handle;
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof Shader other)) return false;
		return this.handle == other.handle;
	}

	@Override
	public int hashCode() {
		return super.hashCode();
	}

	@Override
	public String toString() {
		return STR."Shader[handle=\{this.handle}]";
	}
}
