package dev.pfaff.altargui.shader;

import dev.pfaff.altargui.util.GLThread;
import dev.pfaff.log4truth.Logger;

import java.lang.foreign.Arena;
import java.lang.foreign.MemorySegment;
import java.util.List;

import static dev.pfaff.altargui.GuiGlobals.GL;
import static dev.pfaff.log4truth.StandardTags.DEBUG;

public class ShaderProg {
	private static final String CLASS_NAME = ShaderProg.class.getSimpleName();

	@SuppressWarnings({"FieldCanBeLocal", "unused"})
	private final MemorySegment keepAlive;
	private final String name;
	private final int handle;
	@SuppressWarnings({"FieldCanBeLocal", "unused"})
	private final Shader vertexShader;
	@SuppressWarnings({"FieldCanBeLocal", "unused"})
	private final Shader fragmentShader;

	public ShaderProg(String name, int handle, Shader vertexShader, Shader fragmentShader) {
		Logger.log(() -> STR."Created shader program '\{name}' (\{handle}) with \{vertexShader} and \{fragmentShader}", List.of(DEBUG));
		this.name = name;
		this.handle = handle;
		this.vertexShader = vertexShader;
		this.fragmentShader = fragmentShader;
		this.keepAlive = MemorySegment.NULL.reinterpret(Arena.ofAuto(), _ -> {
			Logger.log(() -> STR."Freeing shader program '\{name}' (\{handle})", List.of(DEBUG));
			GLThread.submitToGLThread(() -> GL.glDeleteProgram(handle));
		});
	}

	public final String name() {
		return this.name;
	}

	public final int handle() {
		return this.handle;
	}

	/**
	 * @throws IllegalArgumentException if the uniform cannot be located
	 */
	public final UniformHandle locateUniform(String name) {
		int location = GL.glGetUniformLocation(this.handle, name);
		if (location == -1) {
			throw new IllegalArgumentException(STR."The shader \{this} does not have a uniform named '\{name}'");
		}
		return new UniformHandle(this, location);
	}

	@Override
	public String toString() {
		return STR."\{CLASS_NAME}[name='\{this.name}', program=\{this.handle}]";
	}

	public void bind() {
		GL.glUseProgram(this.handle);
	}
}
