package dev.pfaff.altargui.shader;

import dev.pfaff.glass.opengl.enums.GLVertexAttribPointerType;
import dev.pfaff.glass.opengl.enums.IGLVertexAttribPointerType;

import java.nio.ByteOrder;

public record VertexAttribSpec(String name, IGLVertexAttribPointerType type, boolean normalized, int width, ByteOrder byteOrder) {
	public static final VertexAttribSpec PADDING_BYTE = new VertexAttribSpec("padding", GLVertexAttribPointerType.BYTE, false, 1);

	public VertexAttribSpec {
		if (normalized && !(type instanceof GLVertexAttribPointerType)) {
			throw new UnsupportedOperationException("Only GLVertexAttribPointerType can be normalized");
		}
	}

	public VertexAttribSpec(String name, IGLVertexAttribPointerType type, boolean normalized, int width) {
		this(name, type, normalized, width, ByteOrder.nativeOrder());
	}

	public VertexAttribSpec(String name, IGLVertexAttribPointerType type, int width) {
		this(name, type, false, width);
	}

	public VertexAttribSpec withNormalized() {
		return new VertexAttribSpec(name, type, true, width, byteOrder);
	}

	public VertexAttribSpec withByteOrder(ByteOrder byteOrder) {
		return new VertexAttribSpec(name, type, normalized, width, byteOrder);
	}
}
