package dev.pfaff.altargui.text;

import dev.pfaff.altargui.util.FakeConstantBoolean;
import dev.pfaff.altargui.util.ThreadFuture;
import dev.pfaff.altargui.util.listenable.EventListenable;
import org.jspecify.annotations.Nullable;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.WeakHashMap;

import static dev.pfaff.altargui.GuiGlobals.LOGGER;
import static dev.pfaff.altargui.util.ThreadFuture.spawnVirtualTask;
import static dev.pfaff.log4truth.StandardTags.ERROR;

public final class FontManager {
	private static final Map<Font, @Nullable Void> FONTS = new WeakHashMap<>();
	private static final FakeConstantBoolean loaded = new FakeConstantBoolean();
	public static final EventListenable ON_RELOAD = new EventListenable();

	public static void load(Font font) {
		Objects.requireNonNull(font);

		boolean needsInstantLoad;
		synchronized (FONTS) {
			if (FONTS.containsKey(font)) return;

			FONTS.put(font, null);

			needsInstantLoad = loaded.get();
		}
		if (needsInstantLoad) {
			try {
				font.reload();
			} catch (Throwable e) {
				LOGGER.log(() -> "Failed to load font", e, List.of(ERROR));
			}
		}
	}

	public static void reload() {
		try {
			ThreadFuture<Void>[] futures;
			synchronized (FONTS) {
				futures = FONTS.keySet()
						.stream()
						.map(font -> spawnVirtualTask(font::reload))
						.<ThreadFuture<Void>>toArray(ThreadFuture[]::new);
				// set loaded while we have FONTS synchronized so that load(Font) will load itself if necessary
				loaded.set(true);
			}
			for (var future : futures) future.join();
		} catch (Throwable e) {
			LOGGER.log(() -> "Failed to load fonts", e, List.of(ERROR));
			throw e;
		}
		ON_RELOAD.notifyListeners();
	}

	public static boolean isInitialized() {
		return loaded.get();
	}
}
