package dev.pfaff.altargui.text;

import java.io.IOException;
import java.lang.foreign.MemorySegment;
import java.lang.foreign.ValueLayout;
import java.nio.file.Files;
import java.nio.file.Path;

public record SDF(MemorySegment pixels, int width, int height) {
	public SDF {
		if (width < 0) throw new IllegalArgumentException("Width cannot be negative");
		if (height < 0) throw new IllegalArgumentException("Height cannot be negative");
		if (pixels.byteSize() != (long) width * (long) height) {
			throw new IllegalStateException("Pixel buffer size does not match dimensions");
		}
		if (pixels.byteSize() > Integer.MAX_VALUE) throw new UnsupportedOperationException();
	}

	public static SDF load(Path path, int width, int height) throws IOException {
		return new SDF(MemorySegment.ofArray(Files.readAllBytes(path)), width, height);
	}

	public int findCharacterEndX(int cellWidth, int cellHeight, int cellX, int cellY) {
		int xLwr = cellY * this.width + cellX;
		int xLwrUpr = xLwr + cellHeight * this.width;
		int endX = 0;
		for (; xLwr < xLwrUpr; xLwr += this.width) {
			int rowLwr = xLwr + endX;
			for (int x = xLwr + cellWidth; --x > rowLwr;) {
				if (this.pixels.get(ValueLayout.JAVA_BYTE, x) != 0) {
					assert endX < x - xLwr : "optimization wrong";
					endX = x - xLwr;
					// end the row
					break;
				}
			}
		}
		return endX;
	}
}
