package dev.pfaff.altargui.texture;

import java.lang.foreign.MemorySegment;

public final class TextureHandle {
	/**
	 * A zero-length memory segment that keeps the texture atlas region alive. The cleanup action will decrement the
	 * ref-count in the allocator.
	 */
	@SuppressWarnings({"FieldCanBeLocal", "unused"})
	private final MemorySegment handle;
	public final int layer;
	public final float u;
	public final float v;
	/**
	 * Multiply your texture UVs ({@code uv / textureSize}) by this factor to get program UVs.
	 */
	public final float scaleFactorU;
	public final float scaleFactorV;

	TextureHandle(MemorySegment handle,
				  int layer,
				  int layerSize,
				  float u,
				  float v,
				  int textureWidth,
				  int textureHeight) {
		this.handle = handle;
		this.layer = layer;
		this.u = u;
		this.v = v;
		this.scaleFactorU = textureWidth / (float)layerSize;
		this.scaleFactorV = textureHeight / (float)layerSize;
	}

	public static TextureHandle unmanaged(int layer,
										  int layerSize,
										  float u,
										  float v,
										  int textureWidth,
										  int textureHeight) {
		return new TextureHandle(MemorySegment.NULL, layer, layerSize, u, v, textureWidth, textureHeight);
	}
}
