package dev.pfaff.altargui.uploadables;

import dev.pfaff.altargui.internal.util.Specialize;
import dev.pfaff.altargui.render.Sink;
import dev.pfaff.altargui.render.ViaIntermediateBuffer;

import java.lang.foreign.StructLayout;
import java.lang.foreign.ValueLayout;
import java.nio.ByteOrder;

import static java.lang.foreign.MemoryLayout.structLayout;
import static java.lang.foreign.MemoryLayout.unionLayout;

public interface Uploadable {
	static final Uploadable[] EMPTY_ARRAY = new Uploadable[0];
	static final StructLayout POSITION_LAYOUT = structLayout(
		ValueLayout.JAVA_FLOAT.withName("x"),
		ValueLayout.JAVA_FLOAT.withName("y")
	).withName("Position");
	static final StructLayout UV_LAYOUT = structLayout(
		ValueLayout.JAVA_SHORT.withName("u"),
		ValueLayout.JAVA_SHORT.withName("v")
	).withName("UV");
	static final ValueLayout POSITION_PACKED_LAYOUT = ValueLayout.JAVA_LONG
		.withByteAlignment(POSITION_LAYOUT.byteAlignment());
	static final ValueLayout UV_PACKED_LAYOUT = ValueLayout.JAVA_INT
		.withByteAlignment(UV_LAYOUT.byteAlignment());
	static final StructLayout VERTEX_LAYOUT = structLayout(
		unionLayout(POSITION_LAYOUT.withName("components"),
					POSITION_PACKED_LAYOUT.withName("packed"))
			.withName("position")
		, unionLayout(UV_LAYOUT.withName("components"),
					UV_PACKED_LAYOUT.withName("packed"))
			.withName("uv")
		, ValueLayout.JAVA_INT.withName("texLayer")
		, ValueLayout.JAVA_INT.withOrder(ByteOrder.BIG_ENDIAN).withName("colorRgba")
	).withName("Vertex");

	/**
	 * The number of bytes that this consumer will provide the sink with upon {@link #uploadTo(Sink)}.
	 */
	int size();

	void uploadTo(Sink sink);

	@Specialize
	default void uSpecUploadAtUnchecked(ViaIntermediateBuffer buffer, long offset) {
		buffer.iSpecUploadAtUnchecked(offset, this);
	}
}
