package dev.pfaff.altargui.uploadables;

import dev.pfaff.altargui.render.Sink;
import dev.pfaff.altargui.types.Color;
import dev.pfaff.altargui.text.Font;

import java.lang.foreign.Arena;

import static dev.pfaff.altargui.render.TextRenderLayerUtil.*;
import static dev.pfaff.altargui.text.StyleFlags.isBold;

public final class UploadableChar implements Uploadable {
	public static final UploadableChar[] EMPTY_ARRAY = new UploadableChar[0];

	private final UploadableRect rect;
	private final UploadableRect boldRect;
	private int flags;

	public UploadableChar(Arena arena) {
		this.rect = new UploadableRect(arena);
		this.boldRect = new UploadableRect(arena);
		this.updatePositionSizeFlags(0f, 0f, DEFAULT_SIZE, 0);
		this.updateColor(DEFAULT_COLOR);
	}

	public void updatePositionSizeFlags(float x, float y, float size, int flags) {
		rect.updatePositionSize(x, y, size, size);
		boldRect.updatePositionSize(x + BOLD_ADVANCE * size, y, size, size);
		this.flags = flags;
	}

	public void updateTexLayer(int layer) {
		rect.updateTexLayer(layer);
		boldRect.updateTexLayer(layer);
	}

	public void updateColor(Color color) {
		rect.updateColor(color);
		boldRect.updateColor(color);
	}

	public void updateChar(char c) {
		this.updateChar(c, 0f, 0f, 1f, 1f);
	}

	public void updateChar(char c, float offsetX, float offsetY, float scaleFactorU, float scaleFactorV) {
		byte i = c <= 255 ? (byte) c : Font.FALLBACK_CHAR;
		float cellSize = 8f / 128f;
		scaleFactorU *= cellSize;
		scaleFactorV *= cellSize;
		int u = i % 16;
		int v = i / 16;
		float uF = offsetX + u * scaleFactorU;
		float vF = offsetY + v * scaleFactorV;
		this.rect.updateUVExtent(uF, vF, scaleFactorU, scaleFactorV);
		this.boldRect.updateUVExtent(uF, vF, scaleFactorU, scaleFactorV);
	}

	@Override
	public int size() {
		return rect.size() + (isBold(flags) ? boldRect.size() : 0);
	}

	@Override
	public void uploadTo(Sink sink) {
		rect.uploadTo(sink);
		if (isBold(flags)) boldRect.uploadTo(sink);
	}
}
