package dev.pfaff.altargui.uploadables;

import dev.pfaff.altargui.render.Sink;
import dev.pfaff.altargui.types.Color;

import java.lang.foreign.Arena;

/**
 * The outlined area, as measured from the inner edges of the outline, will be the specified dimensions.
 */
public final class UploadableOutline implements Uploadable {
	// TODO: store these inline
	private final UploadableRect left;
	private final UploadableRect right;
	private final UploadableRect top;
	private final UploadableRect bottom;

	public UploadableOutline(Arena arena) {
		this.left = new UploadableRect(arena);
		this.right = new UploadableRect(arena);
		this.top = new UploadableRect(arena);
		this.bottom = new UploadableRect(arena);
	}

	public void updatePositionSizeThickness(float x, float y, float width, float height, float thickness) {
		this.top.updatePositionSize(x - thickness, y - thickness, width + thickness * 2, thickness);
		this.bottom.updatePositionSize(x - thickness, y + height, width + thickness * 2, thickness);
		this.left.updatePositionSize(x - thickness, y, thickness, height);
		this.right.updatePositionSize(x + width, y, thickness, height);
	}

	/**
	 * @param color RGBA
	 */
	public void updateColor(Color color) {
		this.left.updateColor(color);
		this.right.updateColor(color);
		this.top.updateColor(color);
		this.bottom.updateColor(color);
	}

	@Override
	public int size() {
		return this.left.size() + this.right.size() + this.top.size() + this.bottom.size();
	}

	@Override
	public void uploadTo(Sink sink) {
		this.left.uploadTo(sink);
		this.right.uploadTo(sink);
		this.top.uploadTo(sink);
		this.bottom.uploadTo(sink);
	}
}
