package dev.pfaff.altargui.util;

import java.util.concurrent.CancellationException;
import java.util.concurrent.CompletionException;

final class CancelOnInterruptFuture<V> implements BasicFuture<V> {
	private final BasicFuture<V> future;

	public CancelOnInterruptFuture(BasicFuture<V> future) {
		this.future = future;
	}

	private V joinThrows() throws InterruptedException {
		return this.future.join();
	}

	@Override
	public V join() throws CompletionException, CancellationException {
		try {
			return joinThrows();
		} catch (InterruptedException e) {
			cancel();
			throw new CancellationException(e.getMessage());
		}
	}

	@Override
	public void cancel() {
		this.future.cancel();
	}

	@Override
	public BasicFuture<V> cancelOnInterrupt() {
		return this;
	}

	@Override
	public BasicFuture<V> uncatch() {
		return this.future.uncatch();
	}
}
