package dev.pfaff.altargui.util;

import dev.pfaff.altargui.internal.util.Specialize;

import java.lang.invoke.MethodHandle;
import java.lang.invoke.MethodHandles;
import java.lang.invoke.MutableCallSite;
import java.util.function.Supplier;

public abstract class FakeConstant<T> implements Supplier<T> {
	private final MutableCallSite CS_value;

	/**
	 * @param type the value type
	 * @param value the initial value
	 */
	public FakeConstant(Class<T> type, T value) {
		this.CS_value = new MutableCallSite(MethodHandles.constant(type, value));
	}

	/**
	 * Stores a new value. This is an expensive operation, so care should be taken by the caller to avoid redundant
	 * invocations.
	 *
	 * @param value the new value
	 */
	public void set(boolean value) {
		try {
			this.CS_value.setTarget(MethodHandles.constant(this.CS_value.type().returnType(), value));
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	protected abstract T implGet(MethodHandle target) throws Throwable;

	/**
	 * @return the stored value
	 */
	@Specialize
	public T get() {
		try {
			return this.implGet(this.CS_value.getTarget());
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}
}
