package dev.pfaff.altargui.util;

import dev.pfaff.altargui.function.FallibleRunnable;

import java.util.concurrent.Callable;
import java.util.concurrent.CompletableFuture;

public final class FutureUtil {
	public static <T> CompletableFuture<T> spawnVirtualThreadFuture(Callable<T> callable) {
		var fut = new CompletableFuture<T>();
		Thread.startVirtualThread(() -> {
			try {
				fut.complete(callable.call());
			} catch (Throwable e) {
				fut.completeExceptionally(e);
			}
		});
		return fut;
	}

	public static CompletableFuture<Void> spawnVirtualThreadFuture(FallibleRunnable runnable) {
		var fut = new CompletableFuture<Void>();
		Thread.startVirtualThread(() -> {
			try {
				runnable.run();
				fut.complete(null);
			} catch (Throwable e) {
				fut.completeExceptionally(e);
			}
		});
		return fut;
	}

	public static void spawnVirtualThread(FallibleRunnable runnable) {
		Thread.startVirtualThread(() -> {
			try {
				runnable.run();
			} catch (Throwable e) {
				throw new RuntimeException(e);
			}
		});
	}
}
