package dev.pfaff.altargui.util;

import java.lang.invoke.MethodHandle;
import java.lang.invoke.MethodHandles;
import java.lang.invoke.MutableCallSite;
import java.util.function.Consumer;

import static java.lang.invoke.MethodType.methodType;

public final class GLThread {
	private static final MutableCallSite SUBMIT_TO_GL_THREAD = new MutableCallSite(methodType(void.class, Runnable.class));
	private static final MethodHandle MH_accept;

	static {
		try {
			MH_accept = MethodHandles.lookup()
					.findVirtual(Consumer.class, "accept", methodType(void.class, Object.class))
					.asType(methodType(void.class, Consumer.class, Runnable.class));
		} catch (ReflectiveOperationException e) {
			throw new RuntimeException(e);
		}
	}

	public static void setSubmitToGLThread(Consumer<Runnable> consumer) {
		SUBMIT_TO_GL_THREAD.setTarget(MH_accept.bindTo(consumer));
	}

	/**
	 * If the current thread is the GL thread, the {@code task} <strong>may</strong> be executed immediately.
	 */
	public static void submitToGLThread(Runnable task) {
		try {
			SUBMIT_TO_GL_THREAD.getTarget().invokeExact(task);
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}
}
