package dev.pfaff.altargui.util;

import org.jetbrains.annotations.Nullable;

import java.nio.charset.StandardCharsets;
import java.util.Objects;

public final class Hasher {
	private long hash;

	public Hasher(long seedHashCode) {
		this.hash = seedHashCode;
	}

	public Hasher() {
		this.hash = 0;
	}

	private static long packString(String s) {
		long l = 0;
		var bs = s.getBytes(StandardCharsets.UTF_8);
		if (bs.length > Long.BYTES) {
			throw new IllegalArgumentException("String is too long to pack: " + s);
		}
		for (byte b : bs) {
			l = l << Byte.SIZE | Byte.toUnsignedLong(b);
		}
		return l;
	}

	public static long packStringAndHash(String s) {
		return hashLong(packString(s));
	}

	public void reset() {
		this.hash = 0;
	}

	public long key() {
		return this.hash;
	}

	private void mixHashCode(long hashCode) {
		this.hash = 31 * this.hash + hashCode;
	}

	public void track(long value) {
		mixHashCode(hashLong(value));
	}

	public void track(int value) {
		mixHashCode(hashInt(value));
	}

	public void track(float value) {
		track(Float.floatToRawIntBits(value));
	}

	public void track(double value) {
		track(Double.doubleToRawLongBits(value));
	}

	public void track(@Nullable Object value) {
		track(Objects.hashCode(value));
	}

	public void trackIdentity(@Nullable Object value) {
		track(System.identityHashCode(value));
	}

	/**
	 * Borrowed from
	 * <a href="https://github.com/skeeto/hash-prospector/tree/396dbe235c94dfc2e9b559fc965bcfda8b6a122c">https://github.com/skeeto/hash-prospector</a>.
	 */
	private static int hashInt(int x) {
		x = (x ^ (x >>> 16)) * 0x21f0aaad;
		x = (x ^ (x >>> 15)) * 0xd35a2d97;
		x = x ^ (x >>> 15);
		return x;
	}

	/**
	 * Borrowed from
	 * <a href="https://stackoverflow.com/a/12996028/10082531">https://stackoverflow.com/a/12996028/10082531</a>.
	 */
	private static long hashLong(long x) {
		x = (x ^ (x >>> 30)) * 0xbf58476d1ce4e5b9L;
		x = (x ^ (x >>> 27)) * 0x94d049bb133111ebL;
		x = x ^ (x >>> 31);
		return x;
	}
}
