package dev.pfaff.altargui.util;

import dev.pfaff.altargui.Key;
import dev.pfaff.altargui.LayoutContext;
import dev.pfaff.altargui.Widget;

import java.util.function.Consumer;
import java.util.function.Supplier;

public final class KeyedWidgetListBuilder {
	private final OpenArrayList<Key.CallSite> keys;
	private final OpenArrayList<Widget> widgets;

	public KeyedWidgetListBuilder(int expectedCapacity) {
		keys = OpenArrayList.wrap(new Key.CallSite[expectedCapacity]);
		widgets = OpenArrayList.wrap(new Widget[expectedCapacity]);
	}

	public KeyedWidgetListBuilder add(Key.CallSite key, Widget widget) {
		keys.add(key);
		widgets.add(widget);
		return this;
	}

	public KeyedWidgetListBuilder addIf(boolean condition, Supplier<Key.CallSite> keySupplier, Supplier<Widget> widgetSupplier) {
		return condition ? add(keySupplier.get(), widgetSupplier.get()) : this;
	}

	public KeyedWidgetListBuilder addIf(boolean condition, Key.CallSite key, Widget widget) {
		return condition ? add(key, widget) : this;
	}

	public Consumer<LayoutContext> build() {
		return mounter -> {
			var keys = this.keys.a();
			var widgets = this.widgets.a();
			int upr = this.keys.size();
			if (this.widgets.size() < upr || keys.length < upr || widgets.length < upr) {
				assert false;
				return;
			}
			for (int i = 0; i < upr; i++) {
				mounter.layout(keys[i], widgets[i]);
			}
		};
	}

	public static Consumer<LayoutContext> buildKeyedWidgetList(int expectedCapacity, Consumer<KeyedWidgetListBuilder> builder) {
		var listBuilder = keyedWidgetListBuilder(expectedCapacity);
		builder.accept(listBuilder);
		return listBuilder.build();
	}

	public static KeyedWidgetListBuilder keyedWidgetListBuilder(int expectedCapacity) {
		return new KeyedWidgetListBuilder(expectedCapacity);
	}
}
