package dev.pfaff.altargui.util;

import dev.pfaff.altargui.Widget;

import java.util.List;
import java.util.function.Consumer;
import java.util.function.Supplier;

public final class ListBuilder<T> {
    private final OpenArrayList<T> list;

    public ListBuilder(int expectedCapacity) {
        list = OpenArrayList.createGeneric(expectedCapacity);
    }

    public ListBuilder<T> add(T value) {
        list.add(value);
        return this;
    }

    public ListBuilder<T> addIf(boolean condition, Supplier<T> valueSupplier) {
        return condition ? add(valueSupplier.get()) : this;
    }

    public ListBuilder<T> addIf(boolean condition, T valueSupplier) {
        return condition ? add(valueSupplier) : this;
    }

    public List<T> build() {
        return List.copyOf(list);
    }

    public static <T> List<T> build(int expectedCapacity, Consumer<ListBuilder<T>> builder) {
        var listBuilder = new ListBuilder<T>(expectedCapacity);
        builder.accept(listBuilder);
        return listBuilder.build();
    }

    public static List<Widget> buildWidgetList(int expectedCapacity, Consumer<ListBuilder<Widget>> builder) {
        return build(expectedCapacity, builder);
    }

    public static ListBuilder<Widget> widgetListBuilder(int expectedCapacity) {
        return new ListBuilder<>(expectedCapacity);
    }
}
