package dev.pfaff.altargui.util;

import dev.pfaff.altargui.LayoutContext;
import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.widgets.MultiChildWidget;

import java.util.List;
import java.util.Objects;
import java.util.function.Consumer;

import static dev.pfaff.altargui.Key.intKey;

public final class ListMultiChildBuilder implements MultiChildWidget.Builder {
	private final List<? extends Widget> widgets;

	private ListMultiChildBuilder(List<? extends Widget> widgets) {
		this.widgets = widgets;
	}

    public static MultiChildWidget.Builder newTrusted(List<? extends Widget> widgets) {
		if (widgets.isEmpty()) {
			// TODO: special case
		}
		if (widgets.size() == 1) {
			return new MultiChildBuilder12(widgets.get(0));
		}
		if (widgets.size() == 2) {
			return new MultiChildBuilder12(widgets.get(0), widgets.get(1));
		}
        return new ListMultiChildBuilder(widgets);
    }

    public static MultiChildWidget.Builder newUntrusted(List<? extends Widget> widgets) {
        return newTrusted(List.copyOf(widgets));
    }

	@Override
	public void build(LayoutContext mounter) {
        this.widgets.forEach(new Consumer<Widget>() {
            private int i = 0;

            @Override
            public void accept(Widget widget) {
                mounter.layout(intKey(this.i++), widget);
            }
        });
	}

	public List<? extends Widget> widgets() {
		return this.widgets;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == this) return true;
		if (obj == null || obj.getClass() != this.getClass()) return false;
		var that = (ListMultiChildBuilder) obj;
		return Objects.equals(this.widgets, that.widgets);
	}

	@Override
	public int hashCode() {
		return this.widgets.hashCode();
	}

	@Override
	public String toString() {
		return STR."ListManagedLayoutContextConsumer[widgets=\{this.widgets}]";
	}
}
