package dev.pfaff.altargui.util;

import it.unimi.dsi.fastutil.chars.CharPredicate;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;

public final class StringCursor {
	public final String string;
	private int offset;

	public StringCursor(String string, int offset) {
		this.string = string;
		this.offset = offset;
	}

	public StringCursor(String string) {
		this.string = string;
	}

	public int offset() {
		return offset;
	}

	public void offset(int offset) {
		if (offset > string.length()) {
			throw new IllegalArgumentException(STR."Cursor offset \{offset} invalid for string of length \{string.length()}");
		}
		this.offset = offset;
	}

	public void advance(int n) {
		offset(offset() + n);
	}

	public boolean matches(String s) {
		return string.regionMatches(offset, s, 0, s.length());
	}

	public boolean advanceMatches(String s) {
		if (matches(s)) {
			advance(s.length());
			return true;
		}
		return false;
	}

	public @Nullable String advanceUntil(char terminator) {
		int i = string.indexOf(offset, terminator);
		if (i == -1) return null;
		var s = string.substring(offset, i);
		offset(i);
		return s;
	}

	public @Nullable String advanceUntil(CharPredicate terminator) {
		int i = nextIndexOf(terminator);
		if (i == -1) return null;
		var s = string.substring(offset, i);
		offset(i);
		return s;
	}

	public @NotNull String advanceUntilOrEnd(CharPredicate terminator) {
		int i = nextIndexOf(terminator);
		if (i == -1) i = string.length();
		var s = string.substring(offset, i);
		offset(i);
		return s;
	}

	public @Nullable String advanceUntilAndSkip(char terminator) {
		var s = advanceUntil(terminator);
		if (s != null) advance(1);
		return s;
	}

	public @Nullable String advanceUntilAndSkip(CharPredicate terminator) {
		var s = advanceUntil(terminator);
		if (s != null) advance(1);
		return s;
	}

	public int nextIndexOf(CharPredicate predicate) {
		for (int i = offset; i < string.length(); i++) {
			if (predicate.test(string.charAt(i))) return i;
		}
		return -1;
	}

	public void skipWhitespace() {
		while (hasRemaining() && Character.isWhitespace(string.charAt(offset))) {
			++offset;
		}
	}

	public boolean hasRemaining() {
		return offset < string.length();
	}

	@Override
	public String toString() {
		return "[" + offset + ": " + string.substring(offset) + ']';
	}
}
