package dev.pfaff.altargui.util;

import dev.pfaff.altargui.function.FallibleRunnable;
import org.jspecify.annotations.Nullable;

import java.util.concurrent.Callable;
import java.util.concurrent.CancellationException;
import java.util.concurrent.CompletionException;

public final class ThreadFuture<V> implements BasicFuture<V> {
	private static final Thread DEAD_THREAD = Thread.startVirtualThread(() -> {});

	private final Thread thread;

	private V value;
	@Nullable
	private Throwable throwable;

	private ThreadFuture(FallibleRunnable runnable) {
		this.thread = Thread.startVirtualThread(() -> {
			try {
				runnable.run();
			} catch (Throwable e) {
				this.throwable = e;
			}
		});
	}

	private ThreadFuture(Callable<V> callable) {
		this.thread = Thread.startVirtualThread(() -> {
			try {
				this.value = callable.call();
			} catch (Throwable e) {
				this.throwable = e;
			}
		});
	}

	public static ThreadFuture<Void> spawnVirtualTask(FallibleRunnable runnable) {
		return new ThreadFuture<>(runnable);
	}

	public static <V> ThreadFuture<V> spawnVirtualTask(Callable<V> callable) {
		return new ThreadFuture<>(callable);
	}

	public V join() throws CompletionException, CancellationException {
		try {
			this.thread.join();
		} catch (InterruptedException e) {
			throw new CancellationException(e.getMessage());
		}
		if (this.throwable != null) throw new CompletionException(this.throwable);
		return this.value;
	}

	@Override
	public void cancel() {
		this.thread.interrupt();
	}

	public boolean isAlive() {
		return this.thread.isAlive();
	}

	public Thread thread() {
		return this.thread;
	}
}
