package dev.pfaff.altargui.util;

import dev.pfaff.altargui.LayoutContext;
import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.widgets.MultiChildWidget;

import java.util.Arrays;
import java.util.Objects;

import static dev.pfaff.altargui.Key.intKey;

public record TrustedArrayMultiChildBuilder<T extends Widget>(T[] widgets, int length) implements MultiChildWidget.Builder {
    public TrustedArrayMultiChildBuilder {
        Objects.checkFromIndexSize(0, length, widgets.length);
    }

    public TrustedArrayMultiChildBuilder(T[] widgets) {
        this(widgets, widgets.length);
    }

    @Override
    public void build(LayoutContext mounter) {
        var widgets = this.widgets;
        var upr = this.length;
        if (upr > widgets.length) throw new AssertionError();
        for (int i = 0; i < upr; i++) {
            Widget widget = this.widgets[i];
            mounter.layout(intKey(i), widget);
        }
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof TrustedArrayMultiChildBuilder<?> other)) return false;
        return this.length == other.length && Arrays.equals(this.widgets, other.widgets);
    }

    @Override
    public int hashCode() {
        return Arrays.hashCode(this.widgets) * 31 + this.length;
    }
}
