package dev.pfaff.altargui.util.listenable;

public final class BooleanSignal extends AbstractSignal {
	private boolean value;

	public BooleanSignal(boolean initialValue) {
		this.value = initialValue;
	}

	private void notifyListeners() {
		notifyListenersAsRunnable();
	}

	public boolean get() {
		return this.value;
	}

	// TODO: make these return whether the value was updated to match the behaviour of WritableValue
	/**
	 * @return the previous value
	 */
	public boolean writeNoNotify(boolean value) {
		var current = this.value;
		if (current != value) this.value = value;
		return current;
	}

	/**
	 * @return the previous value
	 */
	public boolean setNoNotify() {
		return writeNoNotify(true);
	}

	/**
	 * @return the previous value
	 */
	public boolean clearNoNotify() {
		return writeNoNotify(false);
	}

	/**
	 * @return the previous value
	 */
	public boolean write(boolean value) {
		var current = this.value;
		if (current != value) {
			this.value = value;
			notifyListeners();
		}
		return current;
	}

	/**
	 * @return the previous value
	 */
	public boolean set() {
		return write(true);
	}

	/**
	 * @return the previous value
	 */
	public boolean clear() {
		return write(false);
	}

	public void toggle() {
		write(!get());
	}

	public void toggleNoNotify() {
		writeNoNotify(!get());
	}
}
