package dev.pfaff.altargui.util.listenable;

import java.util.function.Consumer;
import java.util.function.Function;

public final class DerivedValue<T, R> implements Value<R> {
	private final Value<T> upstream;
	private final Function<T, R> mapper;

	public DerivedValue(Value<T> upstream, Function<T, R> mapper) {
		this.upstream = upstream;
		this.mapper = mapper;
	}

	@Override
	public ListenerDisposer listen(Runnable listener) {
		return this.upstream.listen(listener);
	}

	@Override
	public R read() {
		return this.mapper.apply(this.upstream.read());
	}

	@Override
	public ListenerDisposer listen(Consumer<R> listener) {
		return this.upstream.listen(t -> listener.accept(this.mapper.apply(t)));
	}

	@Override
	public <R1> Value<R1> map(Function<R, R1> mapper) {
		return new DerivedValue<>(this.upstream, this.mapper.andThen(mapper));
	}
}
