package dev.pfaff.altargui.util.listenable;

import java.lang.invoke.MethodHandles;
import java.lang.invoke.VarHandle;
import java.util.Objects;

public final class SlotValue<T> extends AbstractValue<T> {
	private static final VarHandle VH_value;

	static {
		try {
			VH_value = MethodHandles.lookup().findVarHandle(SlotValue.class, "value", Object.class);
		} catch (NoSuchFieldException | IllegalAccessException e) {
			throw new RuntimeException(e);
		}
	}

	private T value;

	public SlotValue(T initialValue) {
		this.value = initialValue;
	}

	private boolean equals(T a, T b) {
		return Objects.equals(a, b);
	}

	public final T read() {
		return value;
	}

	public final boolean writeNoNotify(T value) {
		if (!equals(this.value, value)) {
			this.value = value;
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Identity is used instead of {@link #equals(Object, Object)}.
	 */
	public final boolean atomicCompareAndSet(T expected, T value) {
		if (VH_value.compareAndSet(this, expected, value)) {
			notifyUpdateValue(value);
			return true;
		}
		return false;
	}
}
