package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.StateContext;
import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.WidgetInstance;
import dev.pfaff.altargui.WidgetInstantiationArgs;
import dev.pfaff.altargui.handlers.MousePressHandler;
import dev.pfaff.altargui.handlers.MouseDragHandler;
import dev.pfaff.altargui.handlers.MouseMoveHandler;
import dev.pfaff.altargui.handlers.MouseReleaseHandler;
import dev.pfaff.altargui.widgets.state.SingleChildWidgetInstance;
import org.jetbrains.annotations.NotNull;

public record DragTarget(Widget child, DragHandler handler) implements Widget {
	@Override
	public @NotNull WidgetInstance<DragTarget> instantiate(WidgetInstantiationArgs args) {
		return new State(args);
	}

	@FunctionalInterface
	public interface DragHandler {
		void handle(boolean dragging, float mouseX, float mouseY, float deltaX, float deltaY);
	}

	private static final class State extends SingleChildWidgetInstance<DragTarget> implements MousePressHandler, MouseReleaseHandler, MouseDragHandler, MouseMoveHandler {
		private DragHandler handler;
		private boolean dragging;

		private State(WidgetInstantiationArgs args) {
			super(args);
		}

		@Override
		public void configure(StateContext stateContext, DragTarget widget, boolean init) {
			setChild(stateContext, widget.child);
			handler = widget.handler;
		}

		@Override
		public boolean mousePressed(double mouseX, double mouseY, int button) {
			if (button == 0 && isInTarget(mouseX, mouseY)) {
				dragging = true;
				handler.handle(true, (float)mouseX, (float)mouseY, 0f, 0f);
				return true;
			}
			return false;
		}

		@Override
		public boolean mouseDragged(double mouseX, double mouseY, int button, double deltaX, double deltaY) {
			if (button == 0 && dragging) {
				handler.handle(true, (float) mouseX, (float) mouseY, (float) deltaX, (float) deltaY);
				return true;
			}
			return false;
		}

		@Override
		public boolean mouseReleased(double mouseX, double mouseY, int button) {
			if (button == 0 && dragging) {
				dragging = false;
				handler.handle(false, (float)mouseX, (float)mouseY, 0f, 0f);
				return true;
			}
			return false;
		}

		@Override
		public boolean mouseMoved(double mouseX, double mouseY) {
			return isInTarget(mouseX, mouseY);
		}

		private boolean isInTarget(double mouseX, double mouseY) {
			return getChildElement().isInside(mouseX, mouseY);
		}
	}
}
