package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.StateContext;
import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.WidgetInstance;
import dev.pfaff.altargui.WidgetInstantiationArgs;
import dev.pfaff.altargui.widgets.state.SingleChildWidgetInstance;
import dev.pfaff.altargui.handlers.MouseMoveHandler;
import it.unimi.dsi.fastutil.booleans.BooleanUnaryOperator;
import org.jetbrains.annotations.NotNull;

/**
 * The return value of {@code handler} determines whether the event will stop bubbling up the widget tree. If {@code
 * true}, the event will stop bubbling. Otherwise, it will continue.
 */
public record MouseMoveTarget(Widget child, BooleanUnaryOperator handler) implements Widget {
	@Override
	public @NotNull WidgetInstance instantiate(WidgetInstantiationArgs args) {
		return new State(args);
	}

	private static final class State extends SingleChildWidgetInstance<MouseMoveTarget>
			implements MouseMoveHandler {
		private BooleanUnaryOperator handler;
		private boolean hovered;

		private State(WidgetInstantiationArgs args) {
			super(args);
		}

		@Override
		public void configure(StateContext stateContext, MouseMoveTarget widget, boolean init) {
			setChild(stateContext, widget.child);
			this.handler = widget.handler;
		}

		@Override
		protected void computeStencilLayout(StateContext context) {
			super.computeStencilLayout(context);
			if (!this.needsTraceLayout.get()) {
				this.mouseMoved(guiContext().lastMouseX(), guiContext().lastMouseY());
			}
		}

		@Override
		protected void computeTraceLayout() {
			super.computeTraceLayout();
			this.mouseMoved(guiContext().lastMouseX(), guiContext().lastMouseY());
		}

		@Override
		public boolean mouseMoved(double mouseX, double mouseY) {
			return mouseMoved((float) mouseX, (float) mouseY);
		}

		private boolean mouseMoved(float mouseX, float mouseY) {
			var hovered = isInside(mouseX, mouseY);
			if (hovered != this.hovered) {
				return this.handler.apply(this.hovered = hovered);
			}
			return false;
		}
	}
}
