package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.StateContext;
import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.WidgetInstance;
import dev.pfaff.altargui.WidgetInstantiationArgs;
import dev.pfaff.altargui.widgets.state.StatefulWidgetInstance;
import dev.pfaff.altargui.handlers.MouseMoveHandler;
import org.jetbrains.annotations.NotNull;

public record MouseOverWithCoordsBuilder(Builder builder) implements Widget {
	@Override
	public @NotNull WidgetInstance<MouseOverWithCoordsBuilder> instantiate(WidgetInstantiationArgs args) {
		return new State(args);
	}

	@FunctionalInterface
	public interface Builder {
		Widget build(boolean hovered, float mouseX, float mouseY);
	}

	private static final class State extends StatefulWidgetInstance<MouseOverWithCoordsBuilder>
			implements MouseMoveHandler {
		private Builder builder;
		private boolean hovered;
		private float mouseX;
		private float mouseY;

		private State(WidgetInstantiationArgs args) {
			super(args);
		}

		@Override
		public void configure(StateContext stateContext, MouseOverWithCoordsBuilder widget, boolean init) {
			if (!widget.builder.equals(this.builder)) {
				this.builder = widget.builder;
				markNeedsRebuild();
			}
		}

		@Override
		protected @NotNull Widget build() {
			return this.builder.build(this.hovered, this.mouseX, this.mouseY);
		}

		@Override
		protected void computeStencilLayout(StateContext context) {
			super.computeStencilLayout(context);
			if (!this.needsTraceLayout.get()) {
				this.mouseMoved(guiContext().lastMouseX(), guiContext().lastMouseY());
			}
		}

		@Override
		protected void computeTraceLayout() {
			super.computeTraceLayout();
			this.mouseMoved(guiContext().lastMouseX(), guiContext().lastMouseY());
		}

		@Override
		public boolean mouseMoved(double mouseX, double mouseY) {
			return mouseMoved((float) mouseX, (float) mouseY);
		}

		private boolean mouseMoved(float mouseX, float mouseY) {
			var hovered = isInside(mouseX, mouseY);
			if (hovered != this.hovered || hovered) {
				this.hovered = hovered;
				this.mouseX = mouseX;
				this.mouseY = mouseY;
				markNeedsRebuild();
				return hovered;
			}
			return false;
		}
	}
}
