package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.StateContext;
import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.WidgetInstantiationArgs;
import org.jetbrains.annotations.NotNull;

import java.util.function.Supplier;

/**
 * The {@code builder} will only be called when the widget is instantiated. It is not tracked. Thus, if you ever
 * need to refresh, you will need to key the widget with the whatever input should trigger a refresh.
 */
public record OnceWidget(Supplier<Widget> builder) implements SimpleBuiltWidget {
	@Override
	public Widget build() {
		return this.builder.get();
	}

	@Override
	public @NotNull AbstractState<OnceWidget> instantiate(WidgetInstantiationArgs args) {
		return new State(args);
	}

	private static final class State extends AbstractState<OnceWidget> {
		private State(WidgetInstantiationArgs args) {
			super(args);
		}

		@Override
		protected boolean update(OnceWidget widget) {
			return false;
		}

		@Override
		public void configure(StateContext stateContext, OnceWidget widget, boolean init) {
			super.configure(stateContext, widget, init);
		}
	}
}
