package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.*;
import dev.pfaff.altargui.widgets.state.Painter;
import dev.pfaff.altargui.inspector.DebugLayer;
import dev.pfaff.altargui.internal.util.StencilLayoutContext;
import dev.pfaff.altargui.render.objects.DebugRectRenderObject;
import org.jetbrains.annotations.NotNull;

public record PaddedWidget(float startX, float startY, float endX, float endY, Widget child) implements SingleChildWidget {
	public static final DebugLayer DEBUG_LAYER_WHOLE = new DebugLayer(PaddedWidget.class, "whole");
	public static final DebugLayer DEBUG_LAYER_INSIDE = new DebugLayer(PaddedWidget.class, "inside");

	@Override
	public @NotNull AbstractState<PaddedWidget> instantiate(WidgetInstantiationArgs args) {
		return new State(args);
	}

	@Override
	public PaddedWidget withMargin(float startX, float startY, float endX, float endY) {
		return new PaddedWidget(this.startX + startX,
				this.startY + startY,
				this.endX + endX,
				this.endY + endY,
				child);
	}

	private static final class State extends AbstractState<PaddedWidget> implements Painter {
		private final DebugRectRenderObject debugWhole;
		private final DebugRectRenderObject debugInside;

		private float startX;
		private float startY;
		private float endX;
		private float endY;

		private State(WidgetInstantiationArgs args) {
			super(args);
			debugWhole = new DebugRectRenderObject();
			debugInside = new DebugRectRenderObject();
		}

		@Override
		public void configure(StateContext stateContext, PaddedWidget widget, boolean init) {
			super.configure(stateContext, widget, init);
			this.startX = widget.startX;
			this.startY = widget.startY;
			this.endX = widget.endX;
			this.endY = widget.endY;
		}

		@Override
		protected void computeStencilLayout(StateContext context) {
			StencilLayoutContext.updateMountAndLayout(context, this, getChildElement(), (e, ctx) -> {
				e.setAvailableSize(Math.max(allocatedWidth() - startX - endX, 0f),
								   Math.max(allocatedHeight() - startY - endY, 0f));
				e.stencilLayout(ctx);
				float width = startX + e.computedWidth() + endX;
				float height = startY + e.computedHeight() + endY;
				setComputedSize(width, height);
			});
		}

		@Override
		protected void computeTraceLayout() {
			var e = getChildElement();
			e.setPosition(allocatedX() + startX, allocatedY() + startY);
			e.traceLayout();
		}

		@Override
		public void paint(long time, Canvas canvas) {
			if (canvas.isDebugLayerEnabled(DEBUG_LAYER_WHOLE)) {
				debugWhole.updatePositionAndSize(allocatedX(), allocatedY(), computedWidth(), computedHeight());
				canvas.upload(debugWhole);
			}
			if (canvas.isDebugLayerEnabled(DEBUG_LAYER_INSIDE)) {
				debugInside.updatePositionAndSize(allocatedX() + startX,
						allocatedY() + startY,
						Math.max(computedWidth() - startX - endX, 0f),
						Math.max(computedHeight() - startY - endY, 0f));
				canvas.upload(debugInside);
			}
		}
	}
}
