package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.StateContext;
import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.WidgetInstance;
import dev.pfaff.altargui.WidgetInstantiationArgs;
import dev.pfaff.altargui.widgets.state.SingleChildWidgetInstance;
import org.jetbrains.annotations.NotNull;
import org.jspecify.annotations.Nullable;

/**
 * An interface only so that records can implement it.
 */
public interface SimpleBuiltWidget extends Widget {
    /**
     * <strong>PROTECTED</strong>
     */
    Widget build();

    /**
	 * <strong>FINAL</strong>
     */
    @Override
    default @NotNull WidgetInstance<? extends SimpleBuiltWidget> instantiate(WidgetInstantiationArgs args) {
        return new State<>(args);
    }

    abstract class AbstractState<W extends SimpleBuiltWidget> extends SingleChildWidgetInstance<W> {
        public AbstractState(WidgetInstantiationArgs args) {
            super(args);
        }

        /**
         * @return {@code true} if the widget has changed, {@code false} otherwise.
         */
        protected abstract boolean update(W widget);

        @Override
        public void configure(StateContext stateContext, W widget, boolean init) {
            if (update(widget) || init) {
                this.setChild(stateContext, widget.build());
            }
        }
    }

    final class State<W extends SimpleBuiltWidget> extends AbstractState<W> {
        @Nullable
        private W previous = null;

        private State(WidgetInstantiationArgs args) {
            super(args);
        }

        @Override
        protected boolean update(W widget) {
            if (!widget.equals(this.previous)) {
                this.previous = widget;
                return true;
            }
            return false;
        }
    }
}
