package dev.pfaff.altargui.widgets.state;

import dev.pfaff.altargui.*;
import dev.pfaff.altargui.inspector.InspectorMeta;
import dev.pfaff.altargui.internal.util.StencilLayoutContext;
import dev.pfaff.altargui.selector.BuiltinSelector;
import dev.pfaff.altargui.selector.Children;
import dev.pfaff.altargui.selector.NthElement;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;
import java.util.function.Consumer;

public abstract class SingleChildWidgetInstance<W extends Widget> extends WidgetInstance<W> {
	@InspectorMeta(tooltip = "The currently mounted child element")
	private WidgetInstance childElement;

	protected SingleChildWidgetInstance(WidgetInstantiationArgs args) {
		super(args);
	}

	protected final void setChild(StateContext stateContext, @NotNull Widget child) {
		setChild(stateContext, child, true);
	}

	protected Key.CallSite childKey() {
		return () -> {};
	}

	protected final void setChild(StateContext stateContext, @NotNull Widget child, boolean notify) {
		var element = stateContext.getState(this, this.childKey(), Objects.requireNonNull(child));
		if (this.childElement != element) {
			this.childElement = element;
			if (notify) {
				markNeedsStencilLayout();
			}
			markNeedsTraceLayout();
			markNeedsScissorLayout();
		}
	}

	protected final WidgetInstance<?> getChildElement() {
		var childElement = this.childElement;
		assert childElement != null : STR."child of \{toStringShort()} is not initialized:\n" + this;
		return childElement;
	}

	@Override
	protected void computeStencilLayout(StateContext context) {
		StencilLayoutContext.updateMountAndLayout(context, this, getChildElement(), (e, ctx) -> {
			e.copyAvailableSizeFrom(this);
			e.stencilLayout(ctx);
			copyComputedSizeFrom(e);
		});
	}

	@Override
	protected void computeTraceLayout() {
		var e = getChildElement();
		e.copyPositionFrom(this);
		e.traceLayout();
	}

	@Override
	protected void computeScissorLayout() {
		this.childElement.copyClippedRectFrom(this);
		this.childElement.scissorLayout();
	}

	@Override
	protected void computeMount(MountContext context) {
//		context.mount(getChildElement());
		context.mount(this.childElement);
	}

	@Override
	public void uSpecMount(MountContext context) {
		super.uSpecMount(context);
	}

	@Override
	public void applyBuiltinSelector(BuiltinSelector selector, Consumer<WidgetInstance> consumer) {
		switch (selector) {
			case NthElement nthElement -> {
				if (nthElement.n() == 0 || nthElement.n() == -1) BuiltinSelector.applyNext(getChildElement(),
						nthElement,
						consumer);
			}
			case Children children -> BuiltinSelector.applyNext(getChildElement(), children, consumer);
		}
	}
}
