package dev.pfaff.altargui.widgets.state;

import dev.pfaff.altargui.StateContext;
import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.WidgetInstantiationArgs;
import org.jetbrains.annotations.NotNull;

public abstract class StatefulWidgetInstance<W extends Widget> extends SingleChildWidgetInstance<W> {
	private boolean needsRebuild = true;

	protected StatefulWidgetInstance(WidgetInstantiationArgs args) {
		super(args);
	}

	protected abstract @NotNull Widget build();

	protected final void markNeedsRebuild() {
		this.needsRebuild = true;
		markNeedsStencilLayout();
	}

	@Override
	protected void computeStencilLayout(StateContext context) {
		if (this.needsRebuild) {
			this.needsRebuild = false;
			var child = build();
			//noinspection ConstantValue
			if (child == null) throw new NullPointerException(getClass().getName() + "#build returned null");
			setChild(context, child, false);
		}

		super.computeStencilLayout(context);
	}
}
